@@ -2,7 +2,7 @@
  *
  *  \brief Dragon/CoCo cartridge support.
  *
- *  \copyright Copyright 2005-2024 Ciaran Anscomb
+ *  \copyright Copyright 2005-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -276,6 +276,14 @@ static struct cart_fingerprint cart_special[] = {
 		.no_autorun = 1,
 	},
 
+	{
+		.name = "DOS-400 (Prológica)",
+		.size = 0x2000,
+		.crc32 = 0xe9ad60a0,
+		.type = "rsdos",
+		.no_autorun = 1,
+	},
+
 	// IDE cartridges
 
 	{
@@ -418,10 +426,8 @@ struct cart_config *cart_config_deserialise(struct ser_handle *sh) {
 }
 
 static void deserialise_mpi_slot(struct cart_config *cc, struct ser_handle *sh, unsigned slot) {
-	if (cc->mpi.slot_cart_name[slot]) {
-		free(cc->mpi.slot_cart_name[slot]);
-		cc->mpi.slot_cart_name[slot] = NULL;
-	}
+	free(cc->mpi.slot_cart_name[slot]);
+	cc->mpi.slot_cart_name[slot] = NULL;
 	int tag;
 	while (!ser_error(sh) && (tag = ser_read_tag(sh)) > 0) {
 		switch (tag) {
@@ -495,13 +501,10 @@ struct cart_config *cart_config_by_name(const char *name) {
 			rom_cart_config = cart_config_new();
 			rom_cart_config->name = xstrdup("romcart");
 		}
-		if (rom_cart_config->description) {
-			free(rom_cart_config->description);
-		}
-		if (rom_cart_config->type) {
-			free(rom_cart_config->type);
-			rom_cart_config->type = NULL;
-		}
+		free(rom_cart_config->description);
+		rom_cart_config->description = NULL;
+		free(rom_cart_config->type);
+		rom_cart_config->type = NULL;
 		/* Make up a description from filename */
 		sds tmp_name = sdsnew(name);
 		char *bname = basename(tmp_name);
@@ -522,9 +525,7 @@ struct cart_config *cart_config_by_name(const char *name) {
 			rom_cart_config->description = xstrdup("ROM cartridge");
 		}
 		sdsfree(tmp_name);
-		if (rom_cart_config->rom) {
-			free(rom_cart_config->rom);
-		}
+		free(rom_cart_config->rom);
 		rom_cart_config->rom = xstrdup(name);
 		if (!rom_cart_config->type) {
 			rom_cart_config->type = xstrdup("rom");
@@ -553,6 +554,8 @@ struct cart_config *cart_find_working_dos(struct machine_config *mc) {
 	} else {
 		if (xroar.cfg.becker.prefer && (tmp = romlist_find("@rsdos_becker"))) {
 			cc = cart_config_by_name("becker");
+		} else if (strcmp(mc->name, "cp400") == 0 && (tmp = romlist_find("@cp450"))) {
+			cc = cart_config_by_name("cp450");
 		} else if ((tmp = romlist_find("@rsdos"))) {
 			cc = cart_config_by_name("rsdos");
 		} else if (!xroar.cfg.becker.prefer && (tmp = romlist_find("@rsdos_becker"))) {
@@ -575,10 +578,8 @@ static void cart_config_update_romcart(void) {
 	if (!cc) {
 		return;
 	}
-	if (cc->type) {
-		free(cc->type);
-		cc->type = NULL;
-	}
+	free(cc->type);
+	cc->type = NULL;
 	FILE *fd = fopen(rom_cart_config->rom, "rb");
 	if (fd) {
 		off_t fsize = fs_file_size(fd);
@@ -717,20 +718,13 @@ static void sdsfree_void(void *sptr) {
 }
 
 static void cart_config_free(struct cart_config *cc) {
-	if (cc->name)
-		free(cc->name);
-	if (cc->description)
-		free(cc->description);
-	if (cc->type)
-		free(cc->type);
-	if (cc->rom)
-		free(cc->rom);
-	if (cc->rom2)
-		free(cc->rom2);
+	free(cc->name);
+	free(cc->description);
+	free(cc->type);
+	free(cc->rom);
+	free(cc->rom2);
 	for (int i = 0; i < 4; i++) {
-		if (cc->mpi.slot_cart_name[i]) {
-			free(cc->mpi.slot_cart_name[i]);
-		}
+		free(cc->mpi.slot_cart_name[i]);
 	}
 	slist_free_full(cc->opts, (slist_free_func)sdsfree_void);
 	free(cc);
@@ -788,7 +782,7 @@ struct cart *cart_create(const char *cc_name) {
 _Bool cart_finish(struct part *p) {
 	struct cart *c = (struct cart *)p;
 	if (c->firq_event.next == &c->firq_event) {
-		event_queue(&MACHINE_EVENT_LIST, &c->firq_event);
+		event_queue(&c->firq_event);
 	}
 	return 1;
 }
@@ -935,7 +929,7 @@ void cart_rom_init(struct cart *c) {
 	c->attach = cart_rom_attach;
 	c->detach = cart_rom_detach;
 
-	event_init(&c->firq_event, DELEGATE_AS0(void, do_firq, c));
+	event_init(&c->firq_event, MACHINE_EVENT_LIST, DELEGATE_AS0(void, do_firq, c));
 	c->signal_firq = DELEGATE_DEFAULT1(void, bool);
 	c->signal_nmi = DELEGATE_DEFAULT1(void, bool);
 	c->signal_halt = DELEGATE_DEFAULT1(void, bool);
@@ -974,8 +968,7 @@ void cart_rom_reset(struct cart *c, _Bool hard) {
 void cart_rom_attach(struct cart *c) {
 	struct cart_config *cc = c->config;
 	if (cc->autorun) {
-		c->firq_event.at_tick = event_current_tick + EVENT_MS(100);
-		event_queue(&MACHINE_EVENT_LIST, &c->firq_event);
+		event_queue_dt(&c->firq_event, EVENT_MS(100));
 	} else {
 		event_dequeue(&c->firq_event);
 	}
@@ -990,8 +983,7 @@ static void do_firq(void *data) {
 	static _Bool level = 0;
 	struct cart *c = data;
 	DELEGATE_SAFE_CALL(c->signal_firq, level);
-	c->firq_event.at_tick = event_current_tick + EVENT_MS(100);
-	event_queue(&MACHINE_EVENT_LIST, &c->firq_event);
+	event_queue_dt(&c->firq_event, EVENT_MS(100));
 	level = !level;
 }
 
