@@ -1256,8 +1256,7 @@ static void cpu_cycle(void *sptr, int ncycles, _Bool RnW, uint16_t A) {
 	struct machine_coco3 *mcc3 = sptr;
 	mcc3->cycles -= ncycles;
 	if (mcc3->cycles <= 0) mcc3->CPU->running = 0;
-	event_current_tick += ncycles;
-	event_run_queue(&MACHINE_EVENT_LIST);
+	event_run_queue(MACHINE_EVENT_LIST, ncycles);
 	MC6809_IRQ_SET(mcc3->CPU, mcc3->PIA0->a.irq | mcc3->PIA0->b.irq | mcc3->GIME->IRQ);
 	MC6809_FIRQ_SET(mcc3->CPU, mcc3->PIA1->a.irq | mcc3->PIA1->b.irq | mcc3->GIME->FIRQ);
 
@@ -1274,6 +1273,7 @@ static void cpu_cycle(void *sptr, int ncycles, _Bool RnW, uint16_t A) {
 			bp_wp_write_hook(mcc3->bp_session, A);
 #endif
 	}
+	mcc3->GIME->IL1 = (PIA_VALUE_A(mcc3->PIA0) | 0x80) != 0xff;
 }
 
 static void cpu_cycle_noclock(void *sptr, int ncycles, _Bool RnW, uint16_t A) {
