@@ -2,7 +2,7 @@
  *
  *  \brief Dragon and Tandy Colour Computer machines.
  *
- *  \copyright Copyright 2003-2024 Ciaran Anscomb
+ *  \copyright Copyright 2003-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -185,7 +185,7 @@ static void dragon_config_complete_common(struct machine_config *mc) {
 		mc->vdg_type = VDG_6847;
 
 	if (mc->ram_init == ANY_AUTO) {
-		mc->ram_init = ram_init_pattern;
+		mc->ram_init = mc->ram < 512 ? ram_init_pattern : ram_init_random;
 	}
 
 	if (mc->keymap == ANY_AUTO) {
@@ -258,7 +258,7 @@ static void dragon_verify_ram_size(struct machine_config *mc) {
 	_Bool is_dragon32 = (strcmp(mc->architecture, "dragon32") == 0);
 
 	// Validate requested total RAM
-	if (mc->ram < 4 || mc->ram > 64) {
+	if ((mc->ram < 4 || mc->ram > 64) && mc->ram != 512) {
 		mc->ram = is_dragon32 ? 32 : 64;
 	} else if (mc->ram < 8) {
 		mc->ram = 4;
@@ -268,8 +268,10 @@ static void dragon_verify_ram_size(struct machine_config *mc) {
 		mc->ram = 16;
 	} else if (mc->ram < 64) {
 		mc->ram = 32;
-	} else {
+	} else if (mc->ram < 512) {
 		mc->ram = 64;
+	} else {
+		mc->ram = 512;
 	}
 
 	// Pick RAM org based on requested total RAM if not specified
@@ -280,8 +282,10 @@ static void dragon_verify_ram_size(struct machine_config *mc) {
 			mc->ram_org = RAM_ORG_16Kx1;
 		} else if (mc->ram < 64) {
 			mc->ram_org = RAM_ORG_32Kx1;
-		} else {
+		} else if (mc->ram < 512) {
 			mc->ram_org = RAM_ORG_64Kx1;
+		} else {
+			mc->ram_org = RAM_ORG(19, 19, 0);
 		}
 	}
 }
@@ -432,7 +436,11 @@ static void dragon_initialise_common(struct machine_dragon_common *md, struct ma
 	m->config = mc;
 
 	// SAM
-	part_add_component(&m->part, part_create("SN74LS783", NULL), "SAM");
+	if (mc->ram < 512) {
+		part_add_component(&m->part, part_create("SN74LS783", NULL), "SAM");
+	} else {
+		part_add_component(&m->part, part_create("SAMx8", NULL), "SAM");
+	}
 
 	// CPU
 	part_add_component(&m->part, part_create((mc->cpu == CPU_HD6309) ? "HD6309" : "MC6809", NULL), "CPU");
@@ -492,6 +500,8 @@ static _Bool dragon_finish_common(struct machine_dragon_common *md) {
 		return 0;
 	}
 
+	md->SAM->CPUD = &md->CPU->D;
+
 	// Register as a messenger client
 	md->msgr_client_id = messenger_client_register();
 
@@ -537,7 +547,7 @@ static _Bool dragon_finish_common(struct machine_dragon_common *md) {
 
 	md->SAM->cpu_cycle = DELEGATE_AS3(void, int, bool, uint16, cpu_cycle, md);
 	md->SAM->vdg_update = DELEGATE_AS0(void, mc6847_update, md->VDG);
-	md->CPU->mem_cycle = DELEGATE_AS2(void, bool, uint16, mc6883_mem_cycle, md->SAM);
+	md->CPU->mem_cycle = DELEGATE_AS2(void, bool, uint16, md->SAM->mem_cycle, md->SAM);
 
 	// Breakpoint session
 	md->bp_session = bp_session_new(m);
@@ -626,7 +636,8 @@ static _Bool dragon_finish_common(struct machine_dragon_common *md) {
 		if (!palette) {
 			palette = vdg_palette_by_name("ideal");
 		}
-		DELEGATE_SAFE_CALL(md->vo->set_cmp_lead_lag, 0., 100.);
+		// Lead/lag 90° (Dragon; LM1889N) or 100° (CoCo; MC1372)
+		DELEGATE_SAFE_CALL(md->vo->set_cmp_lead_lag, 0., md->is_dragon ? 90. : 100.);
 		for (int c = 0; c < NUM_VDG_COLOURS; c++) {
 			float y = palette->palette[c].y;
 			float chb = palette->palette[c].chb;
@@ -938,8 +949,10 @@ static void dragon_create_ram(struct machine_dragon_common *md) {
 	unsigned nbanks = mc->ram / bank_size;
 	if (nbanks < 1)
 		nbanks = 1;
-	if (nbanks > 2)
+	if (nbanks > 2 && mc->ram < 512)
 		nbanks = 2;
+	if (nbanks > 32)
+		nbanks = 32;
 
 	for (unsigned i = 0; i < nbanks; i++)
 		ram_add_bank(ram, i);
@@ -1011,7 +1024,7 @@ static void dragon_reset(struct machine *m, _Bool hard) {
 	if (md->cart && md->cart->reset) {
 		md->cart->reset(md->cart, hard);
 	}
-	mc6883_reset(md->SAM);
+	md->SAM->reset(md->SAM);
 	md->CPU->reset(md->CPU);
 	mc6847_reset(md->VDG);
 	tape_reset(md->tape_interface);
@@ -1266,7 +1279,7 @@ static inline void advance_clock(struct machine_dragon_common *md, int ncycles);
 static void read_byte(struct machine_dragon_common *md, unsigned A);
 static void write_byte(struct machine_dragon_common *md, unsigned A);
 
-// The SAM's mc6883_mem_cycle() is set up to call cpu_cycle(), which does the
+// The SAM's mem_cycle() is set up to call cpu_cycle(), which does the
 // following:
 
 // - calls advance_clock() to indicate time has passed
@@ -1303,8 +1316,7 @@ static void cpu_cycle(void *sptr, int ncycles, _Bool RnW, uint16_t A) {
 static inline void advance_clock(struct machine_dragon_common *md, int ncycles) {
 	md->cycles -= ncycles;
 	if (md->cycles <= 0) md->CPU->running = 0;
-	event_current_tick += ncycles;
-	event_run_queue(&MACHINE_EVENT_LIST);
+	event_run_queue(MACHINE_EVENT_LIST, ncycles);
 }
 
 // Common routine called by cpu_cycle() (or override) to access RAM and devices
@@ -1313,7 +1325,7 @@ static inline void advance_clock(struct machine_dragon_common *md, int ncycles)
 static void dragon_cpu_cycle(struct machine_dragon_common *md, _Bool RnW,
 			     uint16_t A, unsigned Zrow, unsigned Zcol) {
 	md->Dread = 0xff;
-	if (RnW) {
+	if (md->SAM->nWE) {
 		if (md->SAM->RAS0) {
 			ram_d8(md->RAM, 1, 0, Zrow, Zcol, &md->Dread);
 		}
@@ -1352,7 +1364,7 @@ static void dragon_cpu_cycle(struct machine_dragon_common *md, _Bool RnW,
 #endif
 	}
 
-	if (!RnW) {
+	if (!md->SAM->nWE) {
 		if (md->SAM->RAS0) {
 			ram_d8(md->RAM, 0, 0, Zrow, Zcol, &md->CPU->D);
 		}
@@ -1446,7 +1458,7 @@ static void vdg_fetch_handler(void *sptr, uint16_t A, int nbytes, uint16_t *dest
 	struct machine_dragon_common *md = sptr;
 	uint16_t attr = (PIA_VALUE_B(md->PIA1) & 0x10) << 6;  // GM0 -> ¬INT/EXT
 	while (nbytes > 0) {
-		int n = mc6883_vdg_bytes(md->SAM, nbytes);
+		int n = md->SAM->vdg_bytes(md->SAM, nbytes);
 		const uint8_t *Vp = ram_a8(md->RAM, 0, md->SAM->Vrow, md->SAM->Vcol);
 		if (dest && Vp) {
 			for (int i = n; i; i--) {
@@ -1471,7 +1483,7 @@ static void vdg_fetch_handler_chargen(void *sptr, uint16_t A, int nbytes, uint16
 	_Bool EnI = pia_vdg_mode & 0x10;
 	uint16_t Aram7 = EnI ? 0x80 : 0;
 	while (nbytes > 0) {
-		int n = mc6883_vdg_bytes(md->SAM, nbytes);
+		int n = md->SAM->vdg_bytes(md->SAM, nbytes);
 		const uint8_t *Vp = ram_a8(md->RAM, 0, md->SAM->Vrow, md->SAM->Vcol);
 		if (dest && Vp) {
 			for (int i = n; i; i--) {
@@ -1503,7 +1515,7 @@ static uint8_t dragon_read_byte(struct machine *m, unsigned A, uint8_t D) {
 	(void)D;
 	struct machine_dragon_common *md = (struct machine_dragon_common *)m;
 	md->clock_inhibit = 1;
-	mc6883_mem_cycle(md->SAM, 1, A);
+	md->SAM->mem_cycle(md->SAM, 1, A);
 	md->clock_inhibit = 0;
 	return md->CPU->D;
 }
@@ -1514,7 +1526,7 @@ static void dragon_write_byte(struct machine *m, unsigned A, uint8_t D) {
 	struct machine_dragon_common *md = (struct machine_dragon_common *)m;
 	md->CPU->D = D;
 	md->clock_inhibit = 1;
-	mc6883_mem_cycle(md->SAM, 0, A);
+	md->SAM->mem_cycle(md->SAM, 0, A);
 	md->clock_inhibit = 0;
 }
 
@@ -1658,7 +1670,7 @@ static void pia1b_control_postwrite(void *sptr) {
 static void vdg_hs(void *sptr, _Bool level) {
 	struct machine_dragon_common *md = sptr;
 	mc6821_set_cx1(&md->PIA0->a, level);
-	mc6883_vdg_hsync(md->SAM, level);
+	md->SAM->vdg_hsync(md->SAM, level);
 	if (!level) {
 		unsigned p1bval = md->PIA1->b.out_source & md->PIA1->b.out_sink;
 		_Bool GM0 = p1bval & 0x10;
@@ -1671,7 +1683,7 @@ static void vdg_hs(void *sptr, _Bool level) {
 static void vdg_hs_pal_coco(void *sptr, _Bool level) {
 	struct machine_dragon_common *md = sptr;
 	mc6821_set_cx1(&md->PIA0->a, !level);
-	mc6883_vdg_hsync(md->SAM, level);
+	md->SAM->vdg_hsync(md->SAM, level);
 	// PAL uses palletised output so this wouldn't technically matter, but
 	// user is able to cycle to a faux-NTSC colourscheme, so update phase
 	// here as in NTSC code:
@@ -1686,7 +1698,7 @@ static void vdg_hs_pal_coco(void *sptr, _Bool level) {
 static void vdg_fs(void *sptr, _Bool level) {
 	struct machine_dragon_common *md = sptr;
 	mc6821_set_cx1(&md->PIA0->b, level);
-	mc6883_vdg_fsync(md->SAM, level);
+	md->SAM->vdg_fsync(md->SAM, level);
 	if (level) {
 		sound_update(md->snd);
 		md->frame--;
@@ -1718,7 +1730,7 @@ static void coco_print_byte(void *sptr) {
 		return;
 	}
 	// Not ROM?
-	if (mc6883_decode(md->SAM, 1, md->CPU->reg_pc) != 2) {
+	if (md->SAM->decode(md->SAM, 1, md->CPU->reg_pc) != 2) {
 		return;
 	}
 	int byte = MC6809_REG_A(md->CPU);
