@@ -2,7 +2,7 @@
  *
  *  \brief Linux evdev joystick module.
  *
- *  \copyright Copyright 2024 Ciaran Anscomb
+ *  \copyright Copyright 2024-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -269,9 +269,8 @@ static void evdev_js_physical_init(void) {
 	// is accessed, but in case that doesn't happen for a while, we queue
 	// an event to do so.  Each poll will requeue this event to fire in
 	// 100ms.
-	event_init(&ctx->ev_update, DELEGATE_AS0(void, evdev_js_update, ctx));
-	ctx->ev_update.at_tick = event_current_tick + EVENT_MS(100);
-	event_queue(&UI_EVENT_LIST, &ctx->ev_update);
+	event_init(&ctx->ev_update, UI_EVENT_LIST, DELEGATE_AS0(void, evdev_js_update, ctx));
+	event_queue_dt(&ctx->ev_update, EVENT_MS(100));
 
 	glob_t globbuf;
 	globbuf.gl_offs = 0;
@@ -849,9 +848,8 @@ static void evdev_js_device_add(struct evdev_js_context *ctx, int evid) {
 			++actual_naxes;
 		}
 	}
-	if (dmctx.axis_flags) {
-		free(dmctx.axis_flags);
-	}
+	free(dmctx.axis_flags);
+	dmctx.axis_flags = NULL;
 
 	// Count the actual number of button controls present (for reporting),
 	// and initialise them.
@@ -863,9 +861,8 @@ static void evdev_js_device_add(struct evdev_js_context *ctx, int evid) {
 			++actual_nbuttons;
 		}
 	}
-	if (dmctx.button_flags) {
-		free(dmctx.button_flags);
-	}
+	free(dmctx.button_flags);
+	dmctx.button_flags = NULL;
 
 	// Always add as a standalone joystick.  First two axes and buttons.
 	{
@@ -997,10 +994,8 @@ static void evdev_js_device_remove(struct evdev_js_context *ctx, int evid) {
 		}
 	}
 	joystick_config_update_menus();
-	if (d->axis_controls) {
-		free(d->axis_controls);
-		d->axis_controls = NULL;
-	}
+	free(d->axis_controls);
+	d->axis_controls = NULL;
 	d->naxes = 0;
 
 	d->evid = -1;
@@ -1158,8 +1153,7 @@ static void evdev_js_update(void *sptr) {
 	}
 
 	// Reschedule this to run again soon
-	ctx->ev_update.at_tick = event_current_tick + EVENT_MS(100);
-	event_queue(&UI_EVENT_LIST, &ctx->ev_update);
+	event_queue_dt(&ctx->ev_update, EVENT_MS(100));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
