@@ -4,6 +4,8 @@
  *
  *  \copyright Copyright 2013-2024 Ciaran Anscomb
  *
+ *  \copyright Copyright 2021 Tormod Volden
+ *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
  *  XRoar is free software; you can redistribute it and/or modify it under the
@@ -716,7 +718,7 @@ static void send_general_registers(struct gdb_interface_private *gip) {
 			 HD6309_REG_F(((struct HD6309 *)gip->cpu)),
 			 ((struct HD6309 *)gip->cpu)->reg_v);
 	} else {
-		strcat(packet, "xxxxxxxxxx");
+		strcpy(packet + 28, "xxxxxxxxxx");
 	}
 	send_packet_string(gip, packet);
 }
@@ -949,7 +951,7 @@ static void general_query(struct gdb_interface_private *gip, char *args) {
 #ifdef WANT_MACHINE_ARCH_DRAGON
 		if (0 == strcmp(query, "sam") && gip->sam) {
 			LOG_MOD_DEBUG_GDB(LOG_GDB_QUERY, "gdb", "query: xroar.sam\n");
-			sprintf(packet, "%04x", mc6883_get_register(gip->sam));
+			sprintf(packet, "%04x", gip->sam->get_register(gip->sam));
 			send_packet(gip, packet, 4);
 		}
 		return;
@@ -973,7 +975,7 @@ static void general_set(struct gdb_interface_private *gip, char *args) {
 		set += 6;
 #ifdef WANT_MACHINE_ARCH_DRAGON
 		if (0 == strcmp(set, "sam") && gip->sam) {
-			mc6883_set_register(gip->sam, hex16(args));
+			gip->sam->set_register(gip->sam, hex16(args));
 			send_packet_string(gip, "OK");
 			return;
 		}
