@@ -2,7 +2,7 @@
  *
  *  \brief GTK+ 2 tape control window.
  *
- *  \copyright Copyright 2011-2024 Ciaran Anscomb
+ *  \copyright Copyright 2011-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -88,10 +88,17 @@ static gchar *ms_to_string(int ms);
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+static void gtk2_tc_dialog_free(void *sptr) {
+	struct uigtk2_dialog *dlg = sptr;
+	event_dequeue(&ev_update_tape_counters);
+	clear_program_list(dlg);
+}
+
 // Create cassettes dialog window
 
 struct uigtk2_dialog *gtk2_tc_dialog_new(struct ui_gtk2_interface *uigtk2) {
 	struct uigtk2_dialog *dlg = uigtk2_dialog_new(uigtk2, "/uk/org/6809/xroar/gtk2/tapecontrol.ui", "tc_window", ui_tag_tape_dialog);
+	dlg->free = DELEGATE_AS0(void, gtk2_tc_dialog_free, dlg);
 
 	// Join each UI group we're interested in
 	ui_messenger_join_group(dlg->msgr_client_id, ui_tag_tape_dialog, MESSENGER_NOTIFY_DELEGATE(tc_ui_state_notify, dlg));
@@ -123,7 +130,7 @@ struct uigtk2_dialog *gtk2_tc_dialog_new(struct ui_gtk2_interface *uigtk2) {
 	uigtk2_signal_connect(uigtk2, "output_insert", "clicked", tc_output_insert, dlg);
 
 	// While window displayed, an event triggers updating tape counters
-	event_init(&ev_update_tape_counters, DELEGATE_AS0(void, update_tape_counters, dlg));
+	event_init(&ev_update_tape_counters, UI_EVENT_LIST, DELEGATE_AS0(void, update_tape_counters, dlg));
 
 	return dlg;
 }
@@ -147,8 +154,7 @@ static void tc_ui_state_notify(void *sptr, int tag, void *smsg) {
 	case ui_tag_tape_dialog:
 		update_program_list(dlg);
 		if (value) {
-			ev_update_tape_counters.at_tick = event_current_tick + EVENT_MS(500);
-			event_queue(&UI_EVENT_LIST, &ev_update_tape_counters);
+			event_queue_dt(&ev_update_tape_counters, EVENT_MS(500));
 		} else {
 			event_dequeue(&ev_update_tape_counters);
 		}
@@ -339,8 +345,7 @@ static void update_tape_counters(void *sptr) {
 		uigtk2_label_set_text(uigtk2, "output_file_time", ms_to_string(opos));
 	}
 
-	ev_update_tape_counters.at_tick = event_current_tick + EVENT_MS(500);
-	event_queue(&UI_EVENT_LIST, &ev_update_tape_counters);
+	event_queue_dt(&ev_update_tape_counters, EVENT_MS(500));
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -393,7 +398,6 @@ static void update_program_list(struct uigtk2_dialog *dlg) {
 				   TC_POSITION, timestr,
 				   TC_FILE_POINTER, file,
 				   -1);
-		free(file);
 	}
 	tape_seek(xroar.tape_interface->tape_input, old_offset, SEEK_SET);
 }
