@@ -2,7 +2,7 @@
  *
  *  \brief GTK+ 2 tape control window.
  *
- *  \copyright Copyright 2011-2023 Ciaran Anscomb
+ *  \copyright Copyright 2011-2024 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -20,13 +20,8 @@
 #define XROAR_GTK2_TAPECONTROL_H_
 
 struct ui_gtk2_interface;
+struct uigtk2_dialog;
 
-void gtk2_create_tc_window(struct ui_gtk2_interface *uigtk2);
-void gtk2_toggle_tc_window(GtkToggleAction *current, gpointer user_data);
-
-void gtk2_update_tape_state(int flags);
-void gtk2_input_tape_filename_cb(struct ui_gtk2_interface *uigtk2, const char *filename);
-void gtk2_output_tape_filename_cb(struct ui_gtk2_interface *uigtk2, const char *filename);
-void gtk2_update_tape_playing(int playing);
+struct uigtk2_dialog *gtk2_tc_dialog_new(struct ui_gtk2_interface *);
 
 #endif
