@@ -79,6 +79,7 @@ static void set_viewport(void *, int vp_w, int vp_h);
 static void notify_frame_rate(void *, _Bool is_60hz);
 
 static void vogtkgl_ui_set_gl_filter(void *, int tag, void *smsg);
+static void vogtkgl_ui_set_vsync(void *, int tag, void *smsg);
 static void vogtkgl_ui_set_fullscreen(void *, int tag, void *smsg);
 static void vogtkgl_ui_set_menubar(void *, int tag, void *smsg);
 
@@ -116,6 +117,7 @@ _Bool gtkgl_vo_init(struct ui_gtk2_interface *uigtk2) {
 
 	vogtkgl->msgr_client_id = messenger_client_register();
 	ui_messenger_join_group(vogtkgl->msgr_client_id, ui_tag_gl_filter, MESSENGER_NOTIFY_DELEGATE(vogtkgl_ui_set_gl_filter, uigtk2));
+	ui_messenger_join_group(vogtkgl->msgr_client_id, ui_tag_vsync, MESSENGER_NOTIFY_DELEGATE(vogtkgl_ui_set_vsync, uigtk2));
 	ui_messenger_join_group(vogtkgl->msgr_client_id, ui_tag_fullscreen, MESSENGER_NOTIFY_DELEGATE(vogtkgl_ui_set_fullscreen, uigtk2));
 	ui_messenger_join_group(vogtkgl->msgr_client_id, ui_tag_menubar, MESSENGER_NOTIFY_DELEGATE(vogtkgl_ui_set_menubar, uigtk2));
 
@@ -289,6 +291,15 @@ static void vogtkgl_ui_set_gl_filter(void *sptr, int tag, void *smsg) {
 	vo_opengl_update_gl_filter(vogl);
 }
 
+static void vogtkgl_ui_set_vsync(void *sptr, int tag, void *smsg) {
+	(void)tag;
+	(void)smsg;
+	struct ui_gtk2_interface *uigtk2 = sptr;
+	struct vo_interface *vo = uigtk2->public.vo_interface;
+
+	vo_gtkgl_set_vsync(uigtk2, vo->vsync ? -1 : 0);
+}
+
 static void vogtkgl_ui_set_fullscreen(void *sptr, int tag, void *smsg) {
 	(void)tag;
 	struct ui_gtk2_interface *uigtk2 = sptr;
@@ -396,8 +407,6 @@ static gboolean configure(GtkWidget *da, GdkEventConfigure *event, gpointer data
 	vo_set_draw_area(vo, 0, 0, draw_allocation.width, draw_allocation.height);
 	vo_opengl_setup_context(vogl);
 
-	vo_gtkgl_set_vsync(uigtk2, -1);
-
 	gdk_gl_drawable_gl_end(gldrawable);
 
 	return 0;
@@ -476,6 +485,13 @@ static void vo_gtkgl_set_vsync(struct ui_gtk2_interface *uigtk2, int val) {
 
 #ifdef HAVE_X11
 
+	GdkGLContext *glcontext = gtk_widget_get_gl_context(uigtk2->drawing_area);
+	GdkGLDrawable *gldrawable = gtk_widget_get_gl_drawable(uigtk2->drawing_area);
+
+	if (!gdk_gl_drawable_gl_begin(gldrawable, glcontext)) {
+		g_assert_not_reached();
+	}
+
 	PFNGLXSWAPINTERVALEXTPROC glXSwapIntervalEXT = (PFNGLXSWAPINTERVALEXTPROC)glXGetProcAddress((const GLubyte *)"glXSwapIntervalEXT");
 	if (glXSwapIntervalEXT) {
 		Display *dpy = gdk_x11_drawable_get_xdisplay(gtk_widget_get_window(uigtk2->drawing_area));
@@ -486,6 +502,7 @@ static void vo_gtkgl_set_vsync(struct ui_gtk2_interface *uigtk2, int val) {
 		if (dpy && win) {
 			LOG_MOD_SUB_DEBUG(3, "gtk2", "vo_gtkgl", "glXSwapIntervalEXT(%p, %lu, %d)\n", dpy, win, val);
 			glXSwapIntervalEXT(dpy, win, val);
+			gdk_gl_drawable_gl_end(gldrawable);
 			return;
 		}
 	}
@@ -496,6 +513,7 @@ static void vo_gtkgl_set_vsync(struct ui_gtk2_interface *uigtk2, int val) {
 	if (glXSwapIntervalMESA) {
 		LOG_MOD_SUB_DEBUG(3, "gtk2", "vo_gtkgl", "glXSwapIntervalMESA(%d)\n", val);
 		glXSwapIntervalMESA(val);
+			gdk_gl_drawable_gl_end(gldrawable);
 		return;
 	}
 
@@ -503,9 +521,12 @@ static void vo_gtkgl_set_vsync(struct ui_gtk2_interface *uigtk2, int val) {
 	if (glXSwapIntervalSGI) {
 		LOG_MOD_SUB_DEBUG(3, "gtk2", "vo_gtkgl", "glXSwapIntervalSGI(%d)\n", val);
 		glXSwapIntervalSGI(val);
+		gdk_gl_drawable_gl_end(gldrawable);
 		return;
 	}
 
+	gdk_gl_drawable_gl_end(gldrawable);
+
 #endif
 
 	LOG_MOD_SUB_DEBUG(3, "gtk2", "vo_gtkgl", "Found no way to set swap interval\n");
