@@ -4,7 +4,7 @@
  *
  *  \copyright Copyright 2015-2019 Alan Cox
  *
- *  \copyright Copyright 2021-2022 Ciaran Anscomb
+ *  \copyright Copyright 2021-2024 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -20,20 +20,19 @@
 
 #include "top-config.h"
 
-#include <stdio.h>
+#include <errno.h>
 #include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <string.h>
-#include <stdlib.h>
-#include <errno.h>
-#include <time.h>
 
 #include "array.h"
 #include "xalloc.h"
 
 #include "blockdev.h"
 #include "ide.h"
+#include "logging.h"
 #include "serialise.h"
 
 #define IDE_IDLE        0
@@ -135,8 +134,8 @@ static void ide_xlate_errno(struct ide_taskfile *t)
 
 static void ide_fault(struct ide_drive *d, const char *p)
 {
-  fprintf(stderr, "ide: %s: %d: %s\n", d->controller->name,
-			(int)(d - d->controller->drive), p);
+  LOG_MOD_WARN("idedisk", "%s: %d: %s\n", d->controller->name,
+               (int)(d - d->controller->drive), p);
 }
 
 /* Disk translation */
@@ -156,7 +155,7 @@ static off_t xlate_block(struct ide_taskfile *t)
   /* Some well known software asks for 0/0/0 when it means 0/0/1. Drives appear
      to interpret sector 0 as sector 1 */
   if (t->lba1 == 0) {
-    fprintf(stderr, "[Bug: request for sector offset 0].\n");
+    LOG_MOD_DEBUG(2, "idedisk", "[Bug: request for sector offset 0].\n");
     t->lba1 = 1;
   }
   cyl = (t->lba3 << 8) | t->lba2;
@@ -225,20 +224,21 @@ static void edd_setup(struct ide_taskfile *tf)
   ready(tf);
 }
 
-static void ide_reset(struct ide_controller *c)
+void ide_reset_drive(struct ide_drive *d)
 {
-  if (c->drive[0].present) {
-    edd_setup(&c->drive[0].taskfile);
+  if (d->present) {
+    edd_setup(&d->taskfile);
     /* A drive could clear busy then set DRDY up to 2 minutes later if its
        mindnumbingly slow to start up ! We don't emulate any of that */
-    c->drive[0].taskfile.status = ST_DRDY;
-    c->drive[0].eightbit = 0;
-  }
-  if (c->drive[1].present) {
-    edd_setup(&c->drive[1].taskfile);
-    c->drive[1].taskfile.status = ST_DRDY;
-    c->drive[1].eightbit = 0;
+    d->taskfile.status = ST_DRDY;
+    d->eightbit = 0;
   }
+}
+
+static void ide_reset(struct ide_controller *c)
+{
+  ide_reset_drive(&c->drive[0]);
+  ide_reset_drive(&c->drive[1]);
   c->selected = 0;
 }
 
@@ -811,27 +811,24 @@ void ide_write_latched(struct ide_controller *c, uint8_t reg, uint8_t v)
   ide_write16(c, reg, d);
 }
 
+// If there's a bug in this now, it's mine, not Alan's ..ciaran
 static void make_ascii(uint16_t *p, const char *t, int len)
 {
-  int i;
   char *d = (char *)p;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wstringop-truncation"
-  strncpy(d, t, len);
-#pragma GCC diagnostic pop
-
-  for (i = 0; i < len; i += 2) {
-    char c = *d;
-    *d = d[1];
-    d[1] = c;
-    d += 2;
+  for (int i = 0; i < len; i += 2) {
+    char c1 = t ? *t : 0;
+    t = c1 ? t : NULL;
+    char c0 = t ? *(t+1) : 0;
+    t = c0 ? t + 2 : NULL;
+    *(d++) = c0;
+    *(d++) = c1;
   }
 }
 
 static void make_serial(uint16_t *p)
 {
   char buf[21];
-  srand(getpid()^time(NULL));
+  // assume srand() called at startup
   snprintf(buf, 21, "%08d%08d%04d", rand(), rand(), rand());
   make_ascii(p, buf, 20);
 }
@@ -859,7 +856,7 @@ int ide_make_drive(uint8_t type, int fd)
   ident[53] = le16(1);          /* Geometry words are valid */
 
   switch(type) {
-    case ACME_ROADRUNNER:
+    case BD_ACME_ROADRUNNER:
       /* 504MB drive with LBA support */
       c = 1024;
       h = 16;
@@ -868,7 +865,7 @@ int ide_make_drive(uint8_t type, int fd)
       make_ascii(ident + 27, "ACME ROADRUNNER v0.1", 40);
       ident[49] = le16(1 << 9); /* LBA */
       break;
-    case ACME_ULTRASONICUS:
+    case BD_ACME_ULTRASONICUS:
       /* 40MB drive with LBA support */
       c = 977;
       h = 5;
@@ -877,7 +874,7 @@ int ide_make_drive(uint8_t type, int fd)
       make_ascii(ident + 23, "A001.001", 8);
       make_ascii(ident + 27, "ACME ULTRASONICUS AD INFINITUM v0.1", 40);
       break;
-    case ACME_NEMESIS:
+    case BD_ACME_NEMESIS:
       /* 20MB drive with LBA support */
       c = 615;
       h = 4;
@@ -886,7 +883,7 @@ int ide_make_drive(uint8_t type, int fd)
       make_ascii(ident + 23, "A001.001", 8);
       make_ascii(ident + 27, "ACME NEMESIS RIDICULII v0.1", 40);
       break;
-    case ACME_COYOTE:
+    case BD_ACME_COYOTE:
       /* 20MB drive without LBA support */
       c = 615;
       h = 4;
@@ -894,7 +891,7 @@ int ide_make_drive(uint8_t type, int fd)
       make_ascii(ident + 23, "A001.001", 8);
       make_ascii(ident + 27, "ACME COYOTE v0.1", 40);
       break;
-    case ACME_ACCELLERATTI:
+    case BD_ACME_ACCELLERATTI:
       c = 1024;
       h = 16;
       s = 16;
@@ -902,7 +899,7 @@ int ide_make_drive(uint8_t type, int fd)
       make_ascii(ident + 23, "A001.001", 8);
       make_ascii(ident + 27, "ACME ACCELLERATTI INCREDIBILUS v0.1", 40);
       break;
-    case ACME_ZIPPIBUS:
+    case BD_ACME_ZIPPIBUS:
       c = 1024;
       h = 16;
       s = 32;
@@ -944,25 +941,25 @@ int ide_make_drive(uint8_t type, int fd)
 #define IDE_DRIVE_SER_OFFSET (18)
 
 static const struct ser_struct ser_struct_ide_drive[] = {
-	SER_ID_STRUCT_ELEM(1, ser_type_uint16, struct ide_drive, taskfile.data),
-	SER_ID_STRUCT_ELEM(2, ser_type_uint8, struct ide_drive, taskfile.error),
-	SER_ID_STRUCT_ELEM(3, ser_type_uint8, struct ide_drive, taskfile.feature),
-	SER_ID_STRUCT_ELEM(4, ser_type_uint8, struct ide_drive, taskfile.count),
-	SER_ID_STRUCT_ELEM(5, ser_type_uint8, struct ide_drive, taskfile.lba1),
-	SER_ID_STRUCT_ELEM(6, ser_type_uint8, struct ide_drive, taskfile.lba2),
-	SER_ID_STRUCT_ELEM(7, ser_type_uint8, struct ide_drive, taskfile.lba3),
-	SER_ID_STRUCT_ELEM(8, ser_type_uint8, struct ide_drive, taskfile.lba4),
-	SER_ID_STRUCT_ELEM(9, ser_type_uint8, struct ide_drive, taskfile.status),
-	SER_ID_STRUCT_ELEM(10, ser_type_uint8, struct ide_drive, taskfile.command),
-	SER_ID_STRUCT_ELEM(11, ser_type_uint8, struct ide_drive, taskfile.devctrl),
-	SER_ID_STRUCT_ELEM(12, ser_type_bool, struct ide_drive, intrq),
-	SER_ID_STRUCT_ELEM(13, ser_type_bool, struct ide_drive, failed),
-	SER_ID_STRUCT_ELEM(14, ser_type_bool, struct ide_drive, eightbit),
+	SER_ID_STRUCT_ELEM(1, struct ide_drive, taskfile.data),
+	SER_ID_STRUCT_ELEM(2, struct ide_drive, taskfile.error),
+	SER_ID_STRUCT_ELEM(3, struct ide_drive, taskfile.feature),
+	SER_ID_STRUCT_ELEM(4, struct ide_drive, taskfile.count),
+	SER_ID_STRUCT_ELEM(5, struct ide_drive, taskfile.lba1),
+	SER_ID_STRUCT_ELEM(6, struct ide_drive, taskfile.lba2),
+	SER_ID_STRUCT_ELEM(7, struct ide_drive, taskfile.lba3),
+	SER_ID_STRUCT_ELEM(8, struct ide_drive, taskfile.lba4),
+	SER_ID_STRUCT_ELEM(9, struct ide_drive, taskfile.status),
+	SER_ID_STRUCT_ELEM(10, struct ide_drive, taskfile.command),
+	SER_ID_STRUCT_ELEM(11, struct ide_drive, taskfile.devctrl),
+	SER_ID_STRUCT_ELEM(12, struct ide_drive, intrq),
+	SER_ID_STRUCT_ELEM(13, struct ide_drive, failed),
+	SER_ID_STRUCT_ELEM(14, struct ide_drive, eightbit),
 	SER_ID_STRUCT_UNHANDLED(IDE_DRIVE_SER_DATA),
 	SER_ID_STRUCT_UNHANDLED(IDE_DRIVE_SER_DPTR),
-	SER_ID_STRUCT_ELEM(17, ser_type_int, struct ide_drive, state),
+	SER_ID_STRUCT_ELEM(17, struct ide_drive, state),
 	SER_ID_STRUCT_UNHANDLED(IDE_DRIVE_SER_OFFSET),
-	SER_ID_STRUCT_ELEM(19, ser_type_int, struct ide_drive, length),
+	SER_ID_STRUCT_ELEM(19, struct ide_drive, length),
 };
 
 static _Bool ide_drive_read_elem(void *sptr, struct ser_handle *sh, int tag);
@@ -979,8 +976,8 @@ static const struct ser_struct_data ide_drive_ser_struct_data = {
 
 static const struct ser_struct ser_struct_ide_controller[] = {
 	SER_ID_STRUCT_UNHANDLED(IDE_CONTROLLER_SER_DRIVE),
-	SER_ID_STRUCT_ELEM(2, ser_type_int, struct ide_controller, selected),
-	SER_ID_STRUCT_ELEM(3, ser_type_uint16, struct ide_controller, data_latch),
+	SER_ID_STRUCT_ELEM(2, struct ide_controller, selected),
+	SER_ID_STRUCT_ELEM(3, struct ide_controller, data_latch),
 };
 
 static _Bool ide_controller_read_elem(void *sptr, struct ser_handle *sh, int tag);
