@@ -2,7 +2,7 @@
  *
  *  \brief Tandy MC-10 machine.
  *
- *  \copyright Copyright 2021-2024 Ciaran Anscomb
+ *  \copyright Copyright 2021-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -144,8 +144,7 @@ static void mc10_config_complete(struct machine_config *mc) {
 		}
 	}
 	mc->vdg_type = VDG_6847;
-	if (mc->architecture)
-		free(mc->architecture);
+	free(mc->architecture);
 	mc->architecture = xstrdup("mc10");
 
 	if (mc->ram_init == ANY_AUTO) {
@@ -1047,8 +1046,7 @@ static void mc10_mem_cycle(void *sptr, _Bool RnW, uint16_t A) {
 	mp->cycles -= ncycles;
 	if (mp->cycles <= 0)
 		mp->CPU->running = 0;
-	event_current_tick += ncycles;
-        event_run_queue(&MACHINE_EVENT_LIST);
+        event_run_queue(MACHINE_EVENT_LIST, ncycles);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
