@@ -2,7 +2,7 @@
  *
  *  \brief Motorola MC6801/6803 CPU tracing.
  *
- *  \copyright Copyright 2021 Ciaran Anscomb
+ *  \copyright Copyright 2021-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -26,9 +26,9 @@ struct mc6801_trace;
 struct mc6801_trace *mc6801_trace_new(struct MC6801 *cpu);
 void mc6801_trace_free(struct mc6801_trace *tracer);
 
-void mc6801_trace_reset(struct mc6801_trace *tracer);
-void mc6801_trace_byte(struct mc6801_trace *tracer, uint8_t byte, uint16_t pc);
-void mc6801_trace_irq(struct mc6801_trace *tracer, int vector);
-void mc6801_trace_print(struct mc6801_trace *tracer);
+void mc6801_trace_vector(struct mc6801_trace *, uint16_t vec,
+			 unsigned nbytes, uint8_t *bytes);
+void mc6801_trace_instruction(struct mc6801_trace *, uint16_t pc,
+			      unsigned nbytes, uint8_t *bytes);
 
 #endif
