@@ -2,7 +2,7 @@
  *
  *  \brief Hitach HD6309 CPU.
  *
- *  \copyright Copyright 2012-2024 Ciaran Anscomb
+ *  \copyright Copyright 2012-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -53,14 +53,7 @@
 #include "serialise.h"
 
 #ifdef TRACE
-// Tracing supported:
 #include "hd6309_trace.h"
-#define HD6309_TRACE_VECTOR(c) if (UNLIKELY(logging.trace_cpu)) { hd6309_trace_vector((c)->tracer); }
-#define HD6309_TRACE_INSTRUCTION(c) if (UNLIKELY(logging.trace_cpu)) { hd6309_trace_instruction((c)->tracer); }
-#else
-// Tracing not supported - no-op macros:
-#define HD6309_TRACE_VECTOR(c)
-#define HD6309_TRACE_INSTRUCTION(c)
 #endif
 
 #define HD6309_SER_TFM_SRC  (6)
@@ -103,22 +96,15 @@ static void hd6309_reset(struct MC6809 *cpu);
 static void hd6309_run(struct MC6809 *cpu);
 
 /*
- * Data reading & writing
+ * Compute effective address
  */
 
-/* Wrap common fetches */
-
-static uint8_t fetch_byte(struct MC6809 *cpu, uint16_t a);
-static uint16_t fetch_word(struct MC6809 *cpu, uint16_t a);
-
-/* Compute effective address */
-
 static uint16_t ea_direct(struct MC6809 *cpu);
 static uint16_t ea_extended(struct MC6809 *cpu);
 static uint16_t ea_indexed(struct MC6809 *cpu);
 
 /*
- * Interrupt handling
+ * Interrupt handling, hooks
  */
 
 static void push_irq_registers(struct MC6809 *cpu);
@@ -130,19 +116,9 @@ static void instruction_posthook(struct MC6809 *cpu);
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 /*
- * Register handling macros
+ * Additional 6309 register handling macros
  */
 
-#define REG_CC (cpu->reg_cc)
-#define REG_A (MC6809_REG_A(cpu))
-#define REG_B (MC6809_REG_B(cpu))
-#define REG_D (cpu->reg_d)
-#define REG_DP (cpu->reg_dp)
-#define REG_X (cpu->reg_x)
-#define REG_Y (cpu->reg_y)
-#define REG_U (cpu->reg_u)
-#define REG_S (cpu->reg_s)
-#define REG_PC (cpu->reg_pc)
 #define REG_E (HD6309_REG_E(hcpu))
 #define REG_F (HD6309_REG_F(hcpu))
 #define REG_W (hcpu->reg_w)
@@ -159,16 +135,12 @@ static void instruction_posthook(struct MC6809 *cpu);
 // Read Q
 #define RREG_Q ((REG_D << 16) | REG_W)
 
-/* Condition code register macros */
+// Mode register macros
 
-#define CC_E (0x80)
-#define CC_F (0x40)
-#define CC_H (0x20)
-#define CC_I (0x10)
-#define CC_N (0x08)
-#define CC_Z (0x04)
-#define CC_V (0x02)
-#define CC_C (0x01)
+#define MD_D0 (0x80)
+#define MD_IL (0x40)
+#define MD_FM (0x02)
+#define MD_NM (0x01)
 
 // Common operations
 
@@ -177,17 +149,10 @@ static void instruction_posthook(struct MC6809 *cpu);
 #include "mc6809_common.c"
 #include "mc680x/mc680x_ops.c"
 
-/* Mode register macros */
-
-#define MD_D0 (0x80)
-#define MD_IL (0x40)
-#define MD_FM (0x02)
-#define MD_NM (0x01)
-
 #define NATIVE_MODE (REG_MD & MD_NM)
 #define FIRQ_STACK_ALL (REG_MD & MD_FM)
 
-/* ------------------------------------------------------------------------- */
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 static uint16_t *tfm_reg_to_ptr(struct HD6309 *hcpu, unsigned reg) {
 	struct MC6809 *cpu = &hcpu->mc6809;
@@ -216,7 +181,7 @@ static unsigned tfm_ptr_to_reg(struct HD6309 *hcpu, uint16_t *ptr) {
 	return 15;
 }
 
-/* ------------------------------------------------------------------------- */
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 // HD6309 part creation
 
@@ -341,7 +306,7 @@ static void hd6309_reset(struct MC6809 *cpu) {
 	hcpu->state = hd6309_state_reset;
 }
 
-/* Run CPU while cpu->running is true. */
+// Run CPU while cpu->running is true.
 
 static void hd6309_run(struct MC6809 *cpu) {
 	struct HD6309 *hcpu = (struct HD6309 *)cpu;
@@ -359,7 +324,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			cpu->firq_active = 0;
 			cpu->irq_active = 0;
 			hcpu->state = hd6309_state_reset_check_halt;
-			HD6309_TRACE_VECTOR(hcpu);
 			// fall through
 
 		case hd6309_state_reset_check_halt:
@@ -382,30 +346,30 @@ static void hd6309_run(struct MC6809 *cpu) {
 
 		case hd6309_state_label_b:
 			if (UNLIKELY(cpu->nmi_active)) {
-				HD6309_TRACE_VECTOR(hcpu);
 				peek_byte(cpu, REG_PC);
 				peek_byte(cpu, REG_PC);
 				stack_irq_registers(cpu, 1);
 				hcpu->state = hd6309_state_dispatch_irq;
 			} else if (UNLIKELY(!(REG_CC & CC_F) && cpu->firq_active)) {
-				HD6309_TRACE_VECTOR(hcpu);
 				peek_byte(cpu, REG_PC);
 				peek_byte(cpu, REG_PC);
 				stack_irq_registers(cpu, FIRQ_STACK_ALL);
 				hcpu->state = hd6309_state_dispatch_irq;
 			} else if (UNLIKELY(!(REG_CC & CC_I) && cpu->irq_active)) {
-				HD6309_TRACE_VECTOR(hcpu);
 				peek_byte(cpu, REG_PC);
 				peek_byte(cpu, REG_PC);
 				stack_irq_registers(cpu, 1);
 				hcpu->state = hd6309_state_dispatch_irq;
 			} else {
-				HD6309_TRACE_INSTRUCTION(hcpu);
 				hcpu->state = hd6309_state_next_instruction;
 				cpu->page = 0;
 				// Instruction fetch hook called here so that machine
 				// can be stopped beforehand.
 				DELEGATE_SAFE_CALL(cpu->debug_cpu.instruction_hook);
+#ifdef TRACE
+				cpu->trace_pc = cpu->trace_next_pc = REG_PC;
+				cpu->trace_nbytes = 0;
+#endif
 			}
 			continue;
 
@@ -433,16 +397,16 @@ static void hd6309_run(struct MC6809 *cpu) {
 			continue;
 
 		case hd6309_state_sync:
+			cpu->nmi_active = cpu->nmi_latch;
+			cpu->firq_active = cpu->firq_latch;
+			cpu->irq_active = cpu->irq_latch;
+			NVMA_CYCLE;
 			if (cpu->nmi_active || cpu->firq_active || cpu->irq_active) {
 				NVMA_CYCLE;
 				instruction_posthook(cpu);
 				hcpu->state = hd6309_state_label_b;
 				continue;
 			}
-			cpu->nmi_active = cpu->nmi_latch;
-			cpu->firq_active = cpu->firq_latch;
-			cpu->irq_active = cpu->irq_latch;
-			NVMA_CYCLE;
 			if (cpu->halt) {
 				hcpu->state = hd6309_state_sync_check_halt;
 			}
@@ -474,10 +438,13 @@ static void hd6309_run(struct MC6809 *cpu) {
 
 		case hd6309_state_tfm_write:
 			if (cpu->nmi_active) {
+				instruction_posthook(cpu);
 				hcpu->state = hd6309_state_label_b;
 			} else if (!(REG_CC & CC_F) && cpu->firq_active) {
+				instruction_posthook(cpu);
 				hcpu->state = hd6309_state_label_b;
 			} else if (!(REG_CC & CC_I) && cpu->irq_active) {
+				instruction_posthook(cpu);
 				hcpu->state = hd6309_state_label_b;
 			} else {
 				NVMA_CYCLE;
@@ -639,6 +606,11 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x0211:
 				hcpu->state = hd6309_state_next_instruction;
 				cpu->page = 0x200;
+#ifdef TRACE
+				// Page bytes can chain indefinitely, so ensure we
+				// only keep the first in trace buffer:
+				cpu->trace_nbytes = 1;
+#endif
 				continue;
 
 			// 0x11 Page 3
@@ -648,6 +620,11 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x0311:
 				hcpu->state = hd6309_state_next_instruction;
 				cpu->page = 0x300;
+#ifdef TRACE
+				// Page bytes can chain indefinitely, so ensure we
+				// only keep the first in trace buffer:
+				cpu->trace_nbytes = 1;
+#endif
 				continue;
 
 			// 0x12 NOP inherent
@@ -659,10 +636,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x13:
 				if (!NATIVE_MODE)
 					peek_byte(cpu, REG_PC);
-				cpu->nmi_active = cpu->nmi_latch;
-				cpu->firq_active = cpu->firq_latch;
-				cpu->irq_active = cpu->irq_latch;
-				instruction_posthook(cpu);
 				hcpu->state = hd6309_state_sync;
 				continue;
 
@@ -763,7 +736,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 					case 0x1: tmp2 = REG_X; REG_X = tmp1; break;
 					case 0x2: tmp2 = REG_Y; REG_Y = tmp1; break;
 					case 0x3: tmp2 = REG_U; REG_U = tmp1; break;
-					case 0x4: tmp2 = REG_S; REG_S = tmp1; break;
+					case 0x4: tmp2 = REG_S; REG_S = tmp1; cpu->nmi_armed = 1; break;
 					case 0x5: tmp2 = REG_PC; REG_PC = tmp1; break;
 					case 0x6: tmp2 = REG_W; REG_W = tmp1; break;
 					case 0x7: tmp2 = REG_V; REG_V = tmp1; break;
@@ -780,7 +753,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 					case 0x1: REG_X = tmp2; break;
 					case 0x2: REG_Y = tmp2; break;
 					case 0x3: REG_U = tmp2; break;
-					case 0x4: REG_S = tmp2; break;
+					case 0x4: REG_S = tmp2; cpu->nmi_armed = 1; break;
 					case 0x5: REG_PC = tmp2; break;
 					case 0x6: REG_W = tmp2; break;
 					case 0x7: REG_V = tmp2; break;
@@ -829,7 +802,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 					case 0x1: REG_X = tmp1; break;
 					case 0x2: REG_Y = tmp1; break;
 					case 0x3: REG_U = tmp1; break;
-					case 0x4: REG_S = tmp1; break;
+					case 0x4: REG_S = tmp1; cpu->nmi_armed = 1; break;
 					case 0x5: REG_PC = tmp1; break;
 					case 0x6: REG_W = tmp1; break;
 					case 0x7: REG_V = tmp1; break;
@@ -880,7 +853,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x32:
 				REG_S = ea_indexed(cpu);
 				NVMA_CYCLE;
-				cpu->nmi_armed = 1;  // XXX: Really?
+				cpu->nmi_armed = 1;
 				break;
 
 			// 0x33 LEAU indexed
@@ -905,6 +878,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 					if (postbyte & 0x04) { push_s_byte(cpu, REG_B); }
 					if (postbyte & 0x02) { push_s_byte(cpu, REG_A); }
 					if (postbyte & 0x01) { push_s_byte(cpu, REG_CC); }
+					cpu->nmi_armed = 1;
 				}
 				break;
 
@@ -924,6 +898,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 					if (postbyte & 0x40) { REG_U = pull_s_word(cpu); }
 					if (postbyte & 0x80) { REG_PC = pull_s_word(cpu); }
 					peek_byte(cpu, REG_S);
+					cpu->nmi_armed = 1;
 				}
 				break;
 
@@ -959,7 +934,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 					if (postbyte & 0x08) { REG_DP = pull_u_byte(cpu); }
 					if (postbyte & 0x10) { REG_X = pull_u_word(cpu); }
 					if (postbyte & 0x20) { REG_Y = pull_u_word(cpu); }
-					if (postbyte & 0x40) { REG_S = pull_u_word(cpu); }
+					if (postbyte & 0x40) { REG_S = pull_u_word(cpu); cpu->nmi_armed = 1; }
 					if (postbyte & 0x80) { REG_PC = pull_u_word(cpu); }
 					peek_byte(cpu, REG_U);
 				}
@@ -1075,7 +1050,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0xf4: case 0xf5: case 0xf6:
 			case 0xf8: case 0xf9: case 0xfa: case 0xfb: {
 				unsigned tmp1, tmp2;
-				tmp1 = !(op & 0x40) ? REG_A : REG_B;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = byte_immediate(cpu); break;
 				case 1: tmp2 = byte_direct(cpu); break;
@@ -1083,6 +1057,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = byte_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = !(op & 0x40) ? REG_A : REG_B;
 				switch (op & 0xf) {
 				case 0x0: tmp1 = op_sub(cpu, tmp1, tmp2); break; // SUBA, SUBB
 				case 0x1: (void)op_sub(cpu, tmp1, tmp2); break; // CMPA, CMPB
@@ -1108,7 +1083,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x83: case 0x93: case 0xa3: case 0xb3:
 			case 0xc3: case 0xd3: case 0xe3: case 0xf3: {
 				unsigned tmp1, tmp2;
-				tmp1 = REG_D;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1116,6 +1090,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_D;
 				switch (op & 0x40) {
 				default:
 				case 0x00: tmp1 = op_sub16(cpu, tmp1, tmp2); break; // SUBD
@@ -1137,6 +1112,13 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x0383: case 0x0393: case 0x03a3: case 0x03b3:
 			case 0x038c: case 0x039c: case 0x03ac: case 0x03bc: {
 				unsigned tmp1, tmp2;
+				switch ((op >> 4) & 3) {
+				case 0: tmp2 = word_immediate(cpu); break;
+				case 1: tmp2 = word_direct(cpu); break;
+				case 2: tmp2 = word_indexed(cpu); break;
+				case 3: tmp2 = word_extended(cpu); break;
+				default: tmp2 = 0; break;
+				}
 				switch (op & 0x0308) {
 				default:
 				case 0x0000: tmp1 = REG_X; break;
@@ -1145,13 +1127,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 0x0300: tmp1 = REG_U; break;
 				case 0x0308: tmp1 = REG_S; break;
 				}
-				switch ((op >> 4) & 3) {
-				case 0: tmp2 = word_immediate(cpu); break;
-				case 1: tmp2 = word_direct(cpu); break;
-				case 2: tmp2 = word_indexed(cpu); break;
-				case 3: tmp2 = word_extended(cpu); break;
-				default: tmp2 = 0; break;
-				}
 				(void)op_sub16(cpu, tmp1, tmp2);
 				if (!NATIVE_MODE)
 					NVMA_CYCLE;
@@ -1214,6 +1189,12 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x03d7: case 0x03e7: case 0x03f7: {
 				uint16_t ea;
 				uint8_t tmp1;
+				switch ((op >> 4) & 3) {
+				case 1: ea = ea_direct(cpu); break;
+				case 2: ea = ea_indexed(cpu); break;
+				case 3: ea = ea_extended(cpu); break;
+				default: ea = 0; break;
+				}
 				switch (op & 0x0340) {
 				default:
 				case 0x0000: tmp1 = REG_A; break;
@@ -1221,12 +1202,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 0x0300: tmp1 = REG_E; break;
 				case 0x0340: tmp1 = REG_F; break;
 				}
-				switch ((op >> 4) & 3) {
-				case 1: ea = ea_direct(cpu); break;
-				case 2: ea = ea_indexed(cpu); break;
-				case 3: ea = ea_extended(cpu); break;
-				default: ea = 0; break;
-				}
 				store_byte(cpu, ea, tmp1);
 				CLR_NZV;
 				SET_NZ8(tmp1);
@@ -1245,6 +1220,12 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x029f: case 0x02af: case 0x02bf:
 			case 0x02df: case 0x02ef: case 0x02ff: {
 				uint16_t ea, tmp1;
+				switch ((op >> 4) & 3) {
+				case 1: ea = ea_direct(cpu); break;
+				case 2: ea = ea_indexed(cpu); break;
+				case 3: ea = ea_extended(cpu); break;
+				default: ea = 0; break;
+				}
 				switch (op & 0x034e) {
 				default:
 				case 0x000e: tmp1 = REG_X; break;
@@ -1254,12 +1235,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 0x020e: tmp1 = REG_Y; break;
 				case 0x024e: tmp1 = REG_S; break;
 				}
-				switch ((op >> 4) & 3) {
-				case 1: ea = ea_direct(cpu); break;
-				case 2: ea = ea_indexed(cpu); break;
-				case 3: ea = ea_extended(cpu); break;
-				default: ea = 0; break;
-				}
 				CLR_NZV;
 				SET_NZ16(tmp1);
 				store_byte(cpu, ea, tmp1 >> 8);
@@ -1509,7 +1484,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x0281: case 0x0291: case 0x02a1: case 0x02b1:
 			case 0x028b: case 0x029b: case 0x02ab: case 0x02bb: {
 				unsigned tmp1, tmp2;
-				tmp1 = REG_W;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1517,6 +1491,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_W;
 				switch (op & 0xf) {
 				case 0x0: tmp1 = op_sub16(cpu, tmp1, tmp2); break;
 				case 0x1: (void)op_sub16(cpu, tmp1, tmp2); break;
@@ -1541,7 +1516,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x0289: case 0x0299: case 0x02a9: case 0x02b9:
 			case 0x028a: case 0x029a: case 0x02aa: case 0x02ba: {
 				unsigned tmp1, tmp2;
-				tmp1 = REG_D;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1549,6 +1523,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_D;
 				switch (op & 0xf) {
 				case 0x2: tmp1 = op_sbc16(cpu, tmp1, tmp2); break;
 				case 0x4: tmp1 = op_and16(cpu, tmp1, tmp2); break;
@@ -1781,7 +1756,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			case 0x03e0: case 0x03e1: case 0x03e6: case 0x03eb:
 			case 0x03f0: case 0x03f1: case 0x03f6: case 0x03fb: {
 				unsigned tmp1, tmp2;
-				tmp1 = !(op & 0x40) ? REG_E : REG_F;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = byte_immediate(cpu); break;
 				case 1: tmp2 = byte_direct(cpu); break;
@@ -1789,6 +1763,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = byte_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = !(op & 0x40) ? REG_E : REG_F;
 				switch (op & 0xf) {
 				case 0x0: tmp1 = op_sub(cpu, tmp1, tmp2); break; // SUBE, SUBF
 				case 0x1: (void)op_sub(cpu, tmp1, tmp2); break; // CMPE, CMPF
@@ -1805,7 +1780,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 
 			// 0x118d, 0x119d, 0x11ad, 0x11bd DIVD
 			case 0x038d: case 0x039d: case 0x03ad: case 0x03bd: {
-				uint16_t tmp1 = REG_D;
+				uint16_t tmp1;
 				uint8_t tmp2;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = byte_immediate(cpu); break;
@@ -1814,6 +1789,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = byte_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_D;
 				NVMA_CYCLE;
 				NVMA_CYCLE;
 				NVMA_CYCLE;
@@ -1892,7 +1868,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 
 			// 0x118e, 0x119e, 0x11ae, 0x11be DIVQ
 			case 0x038e: case 0x039e: case 0x03ae: case 0x03be: {
-				uint32_t tmp1 = RREG_Q;
+				uint32_t tmp1;
 				uint16_t tmp2;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
@@ -1901,6 +1877,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = RREG_Q;
 				NVMA_CYCLE;
 				NVMA_CYCLE;
 				NVMA_CYCLE;
@@ -1980,7 +1957,6 @@ static void hd6309_run(struct MC6809 *cpu) {
 			// 0x118f, 0x119f, 0x11af, 0x11bf MULD
 			case 0x038f: case 0x039f: case 0x03af: case 0x03bf: {
 				uint16_t tmp1, tmp2;
-				tmp1 = REG_D;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1988,6 +1964,7 @@ static void hd6309_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_D;
 				int16_t stmp1 = *((int16_t *)&tmp1);
 				int16_t stmp2 = *((int16_t *)&tmp2);
 				int32_t result = stmp1 * stmp2;
@@ -2039,51 +2016,23 @@ static void hd6309_run(struct MC6809 *cpu) {
 static void hd6309_set_pc(void *sptr, unsigned pc) {
 	struct HD6309 *hcpu = sptr;
 	struct MC6809 *cpu = &hcpu->mc6809;
-	HD6309_TRACE_INSTRUCTION(hcpu);
 	REG_PC = pc;
+#ifdef TRACE
+	cpu->trace_pc = cpu->trace_next_pc = pc;
+	cpu->trace_nbytes = 0;
+#endif
 	hcpu->state = hd6309_state_next_instruction;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 /*
- * Data reading & writing
+ * Compute effective address
  */
 
-/* Wrap common fetches */
-
-static uint8_t fetch_byte(struct MC6809 *cpu, uint16_t a) {
-	uint8_t v = fetch_byte_notrace(cpu, a);
-#ifdef TRACE
-	struct HD6309 *hcpu = (struct HD6309 *)cpu;
-	if (UNLIKELY(logging.trace_cpu)) {
-		hd6309_trace_byte(hcpu->tracer, v, a);
-	}
-#endif
-	return v;
-}
-
-static uint16_t fetch_word(struct MC6809 *cpu, uint16_t a) {
-#ifndef TRACE
-	return fetch_word_notrace(cpu, a);
-#else
-	struct HD6309 *hcpu = (struct HD6309 *)cpu;
-	if (LIKELY(!logging.trace_cpu)) {
-		return fetch_word_notrace(cpu, a);
-	}
-	unsigned v0 = fetch_byte_notrace(cpu, a);
-	hd6309_trace_byte(hcpu->tracer, v0, a);
-	unsigned v1 = fetch_byte_notrace(cpu, a+1);
-	hd6309_trace_byte(hcpu->tracer, v1, a+1);
-	return (v0 << 8) | v1;
-#endif
-}
-
-/* Compute effective address */
-
 static uint16_t ea_direct(struct MC6809 *cpu) {
 	struct HD6309 *hcpu = (struct HD6309 *)cpu;
-	unsigned ea = REG_DP << 8 | fetch_byte(cpu, REG_PC++);
+	unsigned ea = (REG_DP << 8) | fetch_byte(cpu, REG_PC++);
 	if (!NATIVE_MODE)
 		NVMA_CYCLE;
 	return ea;
@@ -2138,6 +2087,7 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 		NVMA_CYCLE;
 		NVMA_CYCLE;
 	} else switch (postbyte & 0x0f) {
+		default:
 		case 0x00: ea = reg; reg += 1; peek_byte(cpu, REG_PC); NVMA_CYCLE; if (!NATIVE_MODE) NVMA_CYCLE; break;
 		case 0x01: ea = reg; reg += 2; peek_byte(cpu, REG_PC); NVMA_CYCLE; NVMA_CYCLE; if (!NATIVE_MODE) NVMA_CYCLE; break;
 		case 0x02: reg -= 1; ea = reg; peek_byte(cpu, REG_PC); NVMA_CYCLE; if (!NATIVE_MODE) NVMA_CYCLE; break;
@@ -2154,7 +2104,6 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 		case 0x0d: ea = word_immediate(cpu); ea = ea + REG_PC; peek_byte(cpu, REG_PC); NVMA_CYCLE; if (!NATIVE_MODE) { NVMA_CYCLE; NVMA_CYCLE; } break;
 		case 0x0e: ea = reg + REG_W; NVMA_CYCLE; NVMA_CYCLE; break;
 		case 0x0f: ea = word_immediate(cpu); if (!NATIVE_MODE) NVMA_CYCLE; break;
-		default: ea = 0; break;
 	}
 	if (postbyte & 0x10) {
 		ea = fetch_word_notrace(cpu, ea);
@@ -2164,7 +2113,7 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 	case 0: REG_X = reg; break;
 	case 1: REG_Y = reg; break;
 	case 2: REG_U = reg; break;
-	case 3: REG_S = reg; break;
+	case 3: cpu->nmi_armed |= (REG_S != reg); REG_S = reg; break;
 	}
 	return ea;
 }
@@ -2172,7 +2121,7 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 /*
- * Interrupt handling
+ * Interrupt handling, hooks
  */
 
 static void push_irq_registers(struct MC6809 *cpu) {
@@ -2213,11 +2162,26 @@ static void take_interrupt(struct MC6809 *cpu, uint8_t mask, uint16_t vec) {
 	REG_CC |= mask;
 	NVMA_CYCLE;
 	hcpu->state = hd6309_state_irq_reset_vector;
+#ifdef TRACE
+	cpu->trace_next_pc = vec;
+	cpu->trace_nbytes = 0;
+#endif
 	REG_PC = fetch_word(cpu, vec);
 	hcpu->state = hd6309_state_label_a;
 	NVMA_CYCLE;
+#ifdef TRACE
+	if (UNLIKELY(logging.trace_cpu)) {
+		hd6309_trace_vector(hcpu->tracer, vec, cpu->trace_nbytes, cpu->trace_bytes);
+	}
+#endif
 }
 
 static void instruction_posthook(struct MC6809 *cpu) {
+#ifdef TRACE
+	struct HD6309 *hcpu = (struct HD6309 *)cpu;
+	if (UNLIKELY(logging.trace_cpu)) {
+		hd6309_trace_instruction(hcpu->tracer, cpu->trace_pc, cpu->trace_nbytes, cpu->trace_bytes);
+	}
+#endif
 	DELEGATE_SAFE_CALL(cpu->debug_cpu.instruction_posthook);
 }
