@@ -2,7 +2,7 @@
  *
  *  \brief Motorola MC6809 CPU.
  *
- *  \copyright Copyright 2003-2024 Ciaran Anscomb
+ *  \copyright Copyright 2003-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -30,7 +30,10 @@
  *  -  Undocumented 6809 Behaviours, David Banks [hoglet67]
  *     https://github.com/hoglet67/6809Decoder/wiki/Undocumented-6809-Behaviours
  *
- *  -  6809 Exchange and Transfer Opcodes
+ *  -  The Comprehensive Document of 6809 Undocumented and Undefined Behavior,
+ *     David Flamand [dfffffff], https://gitlab.com/dfffffff/6809_undoc
+ *
+ *  -  6809 Exchange and Transfer Opcodes,
  *     http://tlindner.macmess.org/?p=945
  */
 
@@ -60,14 +63,7 @@
 #include "serialise.h"
 
 #ifdef TRACE
-// Tracing supported:
 #include "mc6809_trace.h"
-#define MC6809_TRACE_VECTOR(c) if (UNLIKELY(logging.trace_cpu)) { mc6809_trace_vector((c)->tracer); }
-#define MC6809_TRACE_INSTRUCTION(c) if (UNLIKELY(logging.trace_cpu)) { mc6809_trace_instruction((c)->tracer); }
-#else
-// Tracing not supported - no-op macros:
-#define MC6809_TRACE_VECTOR(c)
-#define MC6809_TRACE_INSTRUCTION(c)
 #endif
 
 static const struct ser_struct ser_struct_mc6809[] = {
@@ -122,22 +118,15 @@ static void mc6809_reset(struct MC6809 *cpu);
 static void mc6809_run(struct MC6809 *cpu);
 
 /*
- * Data reading & writing
+ * Compute effective address
  */
 
-/* Wrap common fetches */
-
-static uint8_t fetch_byte(struct MC6809 *cpu, uint16_t a);
-static uint16_t fetch_word(struct MC6809 *cpu, uint16_t a);
-
-/* Compute effective address */
-
 static uint16_t ea_direct(struct MC6809 *cpu);
 static uint16_t ea_extended(struct MC6809 *cpu);
 static uint16_t ea_indexed(struct MC6809 *cpu);
 
 /*
- * Interrupt handling
+ * Interrupt handling, hooks
  */
 
 static void push_irq_registers(struct MC6809 *cpu);
@@ -151,7 +140,7 @@ static void instruction_posthook(struct MC6809 *cpu);
  * ALU operations
  */
 
-/* Illegal 6809 8-bit arithmetic operations */
+// Illegal 6809 8-bit arithmetic operations
 
 static uint8_t op_discard(struct MC6809 *cpu, uint8_t a, uint8_t b);
 static uint8_t op_xdec(struct MC6809 *cpu, uint8_t in);
@@ -159,32 +148,6 @@ static uint8_t op_xclr(struct MC6809 *cpu, uint8_t in);
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-/*
- * Register handling macros
- */
-
-#define REG_CC (cpu->reg_cc)
-#define REG_A (MC6809_REG_A(cpu))
-#define REG_B (MC6809_REG_B(cpu))
-#define REG_D (cpu->reg_d)
-#define REG_DP (cpu->reg_dp)
-#define REG_X (cpu->reg_x)
-#define REG_Y (cpu->reg_y)
-#define REG_U (cpu->reg_u)
-#define REG_S (cpu->reg_s)
-#define REG_PC (cpu->reg_pc)
-
-/* Condition code register macros */
-
-#define CC_E (0x80)
-#define CC_F (0x40)
-#define CC_H (0x20)
-#define CC_I (0x10)
-#define CC_N (0x08)
-#define CC_Z (0x04)
-#define CC_V (0x02)
-#define CC_C (0x01)
-
 // Common operations
 
 /*
@@ -279,7 +242,7 @@ static void mc6809_reset(struct MC6809 *cpu) {
 	cpu->state = mc6809_state_reset;
 }
 
-/* Run CPU while cpu->running is true. */
+// Run CPU while cpu->running is true.
 
 static void mc6809_run(struct MC6809 *cpu) {
 
@@ -296,7 +259,9 @@ static void mc6809_run(struct MC6809 *cpu) {
 			cpu->firq_active = 0;
 			cpu->irq_active = 0;
 			cpu->state = mc6809_state_reset_check_halt;
-			MC6809_TRACE_VECTOR(cpu);
+#ifdef TRACE
+			cpu->trace_nbytes = 0;
+#endif
 			// fall through
 
 		case mc6809_state_reset_check_halt:
@@ -319,30 +284,30 @@ static void mc6809_run(struct MC6809 *cpu) {
 
 		case mc6809_state_label_b:
 			if (UNLIKELY(cpu->nmi_active)) {
-				MC6809_TRACE_VECTOR(cpu);
 				peek_byte(cpu, REG_PC);
 				peek_byte(cpu, REG_PC);
 				stack_irq_registers(cpu);
 				cpu->state = mc6809_state_dispatch_irq;
 			} else if (UNLIKELY(!(REG_CC & CC_F) && cpu->firq_active)) {
-				MC6809_TRACE_VECTOR(cpu);
 				peek_byte(cpu, REG_PC);
 				peek_byte(cpu, REG_PC);
 				stack_firq_registers(cpu);
 				cpu->state = mc6809_state_dispatch_irq;
 			} else if (UNLIKELY(!(REG_CC & CC_I) && cpu->irq_active)) {
-				MC6809_TRACE_VECTOR(cpu);
 				peek_byte(cpu, REG_PC);
 				peek_byte(cpu, REG_PC);
 				stack_irq_registers(cpu);
 				cpu->state = mc6809_state_dispatch_irq;
 			} else {
-				MC6809_TRACE_INSTRUCTION(cpu);
 				cpu->state = mc6809_state_next_instruction;
 				cpu->page = 0;
 				// Instruction fetch hook called here so that machine
 				// can be stopped beforehand.
 				DELEGATE_SAFE_CALL(cpu->debug_cpu.instruction_hook);
+#ifdef TRACE
+				cpu->trace_pc = cpu->trace_next_pc = REG_PC;
+				cpu->trace_nbytes = 0;
+#endif
 			}
 			continue;
 
@@ -370,16 +335,16 @@ static void mc6809_run(struct MC6809 *cpu) {
 			continue;
 
 		case mc6809_state_sync:
+			cpu->nmi_active = cpu->nmi_latch;
+			cpu->firq_active = cpu->firq_latch;
+			cpu->irq_active = cpu->irq_latch;
+			NVMA_CYCLE;
 			if (cpu->nmi_active || cpu->firq_active || cpu->irq_active) {
 				NVMA_CYCLE;
 				instruction_posthook(cpu);
 				cpu->state = mc6809_state_label_b;
 				continue;
 			}
-			cpu->nmi_active = cpu->nmi_latch;
-			cpu->firq_active = cpu->firq_latch;
-			cpu->irq_active = cpu->irq_latch;
-			NVMA_CYCLE;
 			if (cpu->halt) {
 				cpu->state = mc6809_state_sync_check_halt;
 			}
@@ -548,6 +513,11 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x0211:
 				cpu->state = mc6809_state_next_instruction;
 				cpu->page = 0x200;
+#ifdef TRACE
+				// Page bytes can chain indefinitely, so ensure we
+				// only keep the first in trace buffer:
+				cpu->trace_nbytes = 1;
+#endif
 				continue;
 
 			// 0x11 Page 3
@@ -557,6 +527,11 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x11:
 				cpu->state = mc6809_state_next_instruction;
 				cpu->page = 0x300;
+#ifdef TRACE
+				// Page bytes can chain indefinitely, so ensure we
+				// only keep the first in trace buffer:
+				cpu->trace_nbytes = 1;
+#endif
 				continue;
 
 			// 0x12 NOP inherent
@@ -577,10 +552,6 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x0213:
 			case 0x0313:
 				peek_byte(cpu, REG_PC);
-				cpu->nmi_active = cpu->nmi_latch;
-				cpu->firq_active = cpu->firq_latch;
-				cpu->irq_active = cpu->irq_latch;
-				instruction_posthook(cpu);
 				cpu->state = mc6809_state_sync;
 				continue;
 
@@ -595,6 +566,8 @@ static void mc6809_run(struct MC6809 *cpu) {
 			// 0x16 LBRA relative
 			// 0x1016, 0x1116 - page 2/3 fallthrough, illegal
 			// 0x108d, 0x118d - LBRA observed by darrena on discord
+			// TODO: investigate [dfffffff] claim of destination
+			// opcode affecting behaviour
 			case 0x16:
 			case 0x0216: case 0x028d:
 			case 0x0316: case 0x038d: {
@@ -622,6 +595,8 @@ static void mc6809_run(struct MC6809 *cpu) {
 			// 0x18 Shift CC with mask inherent (illegal)
 			// 0x10xx, 0x11xx - page 2/3 fallthrough, illegal
 			// Behaviour as defined in [hoglet67]
+			// TODO: do we inc PC for operand?
+			// TODO: is this actually mirrored in the other pages?
 			case 0x18:
 			case 0x0218:
 			case 0x0318: {
@@ -696,7 +671,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 					case 0x1: tmp2 = REG_X; REG_X = tmp1; break;
 					case 0x2: tmp2 = REG_Y; REG_Y = tmp1; break;
 					case 0x3: tmp2 = REG_U; REG_U = tmp1; break;
-					case 0x4: tmp2 = REG_S; REG_S = tmp1; break;
+					case 0x4: tmp2 = REG_S; REG_S = tmp1; cpu->nmi_armed = 1; break;
 					case 0x5: tmp2 = REG_PC; REG_PC = tmp1; break;
 					case 0x8: tmp2 = REG_A | 0xff00; REG_A = tmp1; break;
 					case 0x9: tmp2 = REG_B | 0xff00; REG_B = tmp1; break;
@@ -709,7 +684,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 					case 0x1: REG_X = tmp2; break;
 					case 0x2: REG_Y = tmp2; break;
 					case 0x3: REG_U = tmp2; break;
-					case 0x4: REG_S = tmp2; break;
+					case 0x4: REG_S = tmp2; cpu->nmi_armed = 1; break;
 					case 0x5: REG_PC = tmp2; break;
 					case 0x8: REG_A = tmp2; break;
 					case 0x9: REG_B = tmp2; break;
@@ -750,7 +725,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 					case 0x1: REG_X = tmp1; break;
 					case 0x2: REG_Y = tmp1; break;
 					case 0x3: REG_U = tmp1; break;
-					case 0x4: REG_S = tmp1; break;
+					case 0x4: REG_S = tmp1; cpu->nmi_armed = 1; break;
 					case 0x5: REG_PC = tmp1; break;
 					case 0x8: REG_A = tmp1; break;
 					case 0x9: REG_B = tmp1; break;
@@ -801,7 +776,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x0232:
 				REG_S = ea_indexed(cpu);
 				NVMA_CYCLE;
-				cpu->nmi_armed = 1;  // XXX: Really?
+				cpu->nmi_armed = 1;
 				break;
 
 			// 0x33 LEAU indexed
@@ -830,6 +805,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 					if (postbyte & 0x04) { push_s_byte(cpu, REG_B); }
 					if (postbyte & 0x02) { push_s_byte(cpu, REG_A); }
 					if (postbyte & 0x01) { push_s_byte(cpu, REG_CC); }
+					cpu->nmi_armed = 1;
 				}
 				break;
 
@@ -851,6 +827,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 					if (postbyte & 0x40) { REG_U = pull_s_word(cpu); }
 					if (postbyte & 0x80) { REG_PC = pull_s_word(cpu); }
 					peek_byte(cpu, REG_S);
+					cpu->nmi_armed = 1;
 				}
 				break;
 
@@ -890,7 +867,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 					if (postbyte & 0x08) { REG_DP = pull_u_byte(cpu); }
 					if (postbyte & 0x10) { REG_X = pull_u_word(cpu); }
 					if (postbyte & 0x20) { REG_Y = pull_u_word(cpu); }
-					if (postbyte & 0x40) { REG_S = pull_u_word(cpu); }
+					if (postbyte & 0x40) { REG_S = pull_u_word(cpu); cpu->nmi_armed = 1; }
 					if (postbyte & 0x80) { REG_PC = pull_u_word(cpu); }
 					peek_byte(cpu, REG_U);
 				}
@@ -989,6 +966,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 			} break;
 
 			// 0x3e RESET inherent, illegal
+			// NMI not disarmed
 			// [hoglet67] F and I not set
 			case 0x3e:
 				peek_byte(cpu, REG_PC);
@@ -1084,7 +1062,6 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x03f8: case 0x03f9: case 0x03fa: case 0x03fb: {
 
 				unsigned tmp1, tmp2;
-				tmp1 = !(op & 0x40) ? REG_A : REG_B;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = byte_immediate(cpu); break;
 				case 1: tmp2 = byte_direct(cpu); break;
@@ -1092,6 +1069,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 				case 3: tmp2 = byte_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = !(op & 0x40) ? REG_A : REG_B;
 				switch (op & 0xf) {
 				case 0x0: tmp1 = op_sub(cpu, tmp1, tmp2); break; // SUBA, SUBB
 				case 0x1: (void)op_sub(cpu, tmp1, tmp2); break; // CMPA, CMPB
@@ -1118,7 +1096,6 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x83: case 0x93: case 0xa3: case 0xb3:
 			case 0xc3: case 0xd3: case 0xe3: case 0xf3: {
 				unsigned tmp1, tmp2;
-				tmp1 = REG_D;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1126,6 +1103,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_D;
 				switch (op & 0x40) {
 				default:
 				case 0x00: tmp1 = op_sub16(cpu, tmp1, tmp2); break; // SUBD
@@ -1146,6 +1124,13 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x0383: case 0x0393: case 0x03a3: case 0x03b3:
 			case 0x038c: case 0x039c: case 0x03ac: case 0x03bc: {
 				unsigned tmp1, tmp2;
+				switch ((op >> 4) & 3) {
+				case 0: tmp2 = word_immediate(cpu); break;
+				case 1: tmp2 = word_direct(cpu); break;
+				case 2: tmp2 = word_indexed(cpu); break;
+				case 3: tmp2 = word_extended(cpu); break;
+				default: tmp2 = 0; break;
+				}
 				switch (op & 0x0308) {
 				default:
 				case 0x0000: tmp1 = REG_X; break;
@@ -1154,21 +1139,13 @@ static void mc6809_run(struct MC6809 *cpu) {
 				case 0x0300: tmp1 = REG_U; break;
 				case 0x0308: tmp1 = REG_S; break;
 				}
-				switch ((op >> 4) & 3) {
-				case 0: tmp2 = word_immediate(cpu); break;
-				case 1: tmp2 = word_direct(cpu); break;
-				case 2: tmp2 = word_indexed(cpu); break;
-				case 3: tmp2 = word_extended(cpu); break;
-				default: tmp2 = 0; break;
-				}
 				(void)op_sub16(cpu, tmp1, tmp2);
 				NVMA_CYCLE;
 			} break;
 
 			// 0x10c3, 0x10d3, 0x10e3, 0x10f3 XADDD, illegal [hoglet67]
 			case 0x02c3: case 0x02d3: case 0x02e3: case 0x02f3: {
-				unsigned tmp1 = REG_D;
-				unsigned tmp2;
+				unsigned tmp1, tmp2;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1176,14 +1153,14 @@ static void mc6809_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_D;
 				(void)op_add16(cpu, tmp1, tmp2);
 				NVMA_CYCLE;
 			} break;
 
 			// 0x11c3, 0x11d3, 0x11e3, 0x11f3 XADDU, illegal [hoglet67]
 			case 0x03c3: case 0x03d3: case 0x03e3: case 0x03f3: {
-				unsigned tmp1 = REG_U | 0xff00;
-				unsigned tmp2;
+				unsigned tmp1, tmp2;
 				switch ((op >> 4) & 3) {
 				case 0: tmp2 = word_immediate(cpu); break;
 				case 1: tmp2 = word_direct(cpu); break;
@@ -1191,6 +1168,7 @@ static void mc6809_run(struct MC6809 *cpu) {
 				case 3: tmp2 = word_extended(cpu); break;
 				default: tmp2 = 0; break;
 				}
+				tmp1 = REG_U | 0xff00;
 				(void)op_add16(cpu, tmp1, tmp2);
 				NVMA_CYCLE;
 			} break;
@@ -1270,13 +1248,13 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x03d7: case 0x03e7: case 0x03f7: {
 				uint16_t ea;
 				uint8_t tmp1;
-				tmp1 = !(op & 0x40) ? REG_A : REG_B;
 				switch ((op >> 4) & 3) {
 				case 1: ea = ea_direct(cpu); break;
 				case 2: ea = ea_indexed(cpu); break;
 				case 3: ea = ea_extended(cpu); break;
 				default: ea = 0; break;
 				}
+				tmp1 = !(op & 0x40) ? REG_A : REG_B;
 				store_byte(cpu, ea, tmp1);
 				CLR_NZV;
 				SET_NZ8(tmp1);
@@ -1293,6 +1271,12 @@ static void mc6809_run(struct MC6809 *cpu) {
 			case 0x029f: case 0x02af: case 0x02bf:
 			case 0x02df: case 0x02ef: case 0x02ff: {
 				uint16_t ea, tmp1;
+				switch ((op >> 4) & 3) {
+				case 1: ea = ea_direct(cpu); break;
+				case 2: ea = ea_indexed(cpu); break;
+				case 3: ea = ea_extended(cpu); break;
+				default: ea = 0; break;
+				}
 				switch (op & 0x034e) {
 				default:
 				case 0x000e: tmp1 = REG_X; break;
@@ -1301,12 +1285,6 @@ static void mc6809_run(struct MC6809 *cpu) {
 				case 0x020e: tmp1 = REG_Y; break;
 				case 0x024e: tmp1 = REG_S; break;
 				}
-				switch ((op >> 4) & 3) {
-				case 1: ea = ea_direct(cpu); break;
-				case 2: ea = ea_indexed(cpu); break;
-				case 3: ea = ea_extended(cpu); break;
-				default: ea = 0; break;
-				}
 				CLR_NZV;
 				SET_NZ16(tmp1);
 				store_byte(cpu, ea, tmp1 >> 8);
@@ -1390,48 +1368,22 @@ static void mc6809_run(struct MC6809 *cpu) {
 
 static void mc6809_set_pc(void *sptr, unsigned pc) {
 	struct MC6809 *cpu = sptr;
-	MC6809_TRACE_INSTRUCTION(cpu);
 	REG_PC = pc;
+#ifdef TRACE
+	cpu->trace_pc = cpu->trace_next_pc = pc;
+	cpu->trace_nbytes = 0;
+#endif
 	cpu->state = mc6809_state_next_instruction;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 /*
- * Data reading & writing
+ * Compute effective address
  */
 
-/* Wrap common fetches */
-
-static uint8_t fetch_byte(struct MC6809 *cpu, uint16_t a) {
-	uint8_t v = fetch_byte_notrace(cpu, a);
-#ifdef TRACE
-	if (UNLIKELY(logging.trace_cpu)) {
-		mc6809_trace_byte(cpu->tracer, v, a);
-	}
-#endif
-	return v;
-}
-
-static uint16_t fetch_word(struct MC6809 *cpu, uint16_t a) {
-#ifndef TRACE
-	return fetch_word_notrace(cpu, a);
-#else
-	if (LIKELY(!logging.trace_cpu)) {
-		return fetch_word_notrace(cpu, a);
-	}
-	unsigned v0 = fetch_byte_notrace(cpu, a);
-	mc6809_trace_byte(cpu->tracer, v0, a);
-	unsigned v1 = fetch_byte_notrace(cpu, a+1);
-	mc6809_trace_byte(cpu->tracer, v1, a+1);
-	return (v0 << 8) | v1;
-#endif
-}
-
-/* Compute effective address */
-
 static uint16_t ea_direct(struct MC6809 *cpu) {
-	unsigned ea = REG_DP << 8 | fetch_byte(cpu, REG_PC++);
+	unsigned ea = (REG_DP << 8) | fetch_byte(cpu, REG_PC++);
 	NVMA_CYCLE;
 	return ea;
 }
@@ -1443,6 +1395,13 @@ static uint16_t ea_extended(struct MC6809 *cpu) {
 	return ea;
 }
 
+/* Indexed addressing.
+ *
+ * Some illegal behaviour taken from [dfffffff].
+ *
+ * TODO: check which addresses are in play for 1RRI1010 postbyte behaviour
+ * reported by [dfffffff].  Currently only timings are implemented. */
+
 static uint16_t ea_indexed(struct MC6809 *cpu) {
 	unsigned ea;
 	uint16_t reg;
@@ -1460,6 +1419,7 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 		return reg + sex5(postbyte & 0x1f);
 	}
 	switch (postbyte & 0x0f) {
+		default:
 		case 0x00: ea = reg; reg += 1; peek_byte(cpu, REG_PC); NVMA_CYCLE; NVMA_CYCLE; break;
 		case 0x01: ea = reg; reg += 2; peek_byte(cpu, REG_PC); NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; break;
 		case 0x02: reg -= 1; ea = reg; peek_byte(cpu, REG_PC); NVMA_CYCLE; NVMA_CYCLE; break;
@@ -1470,13 +1430,12 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 		case 0x06: ea = reg + sex8(REG_A); peek_byte(cpu, REG_PC); NVMA_CYCLE; break;
 		case 0x08: ea = byte_immediate(cpu); ea = sex8(ea) + reg; NVMA_CYCLE; break;
 		case 0x09: ea = word_immediate(cpu); ea = ea + reg; NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; break;
-		case 0x0a: ea = REG_PC | 0xff; break;
+		case 0x0a: ea = REG_PC | 0xff; peek_byte(cpu, REG_PC); NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; break;  // TODO: see [dfffffff]
 		case 0x0b: ea = reg + REG_D; peek_byte(cpu, REG_PC); peek_byte(cpu, REG_PC + 1); NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; break;
 		case 0x0c: ea = byte_immediate(cpu); ea = sex8(ea) + REG_PC; NVMA_CYCLE; break;
 		case 0x0d: ea = word_immediate(cpu); ea = ea + REG_PC; peek_byte(cpu, REG_PC); NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; break;
-		case 0x0e: ea = 0xffff; break; // illegal
+		case 0x0e: peek_byte(cpu, REG_PC); ea = 0xffff; NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; NVMA_CYCLE; break; // illegal
 		case 0x0f: ea = word_immediate(cpu); NVMA_CYCLE; break;
-		default: ea = 0; break;
 	}
 	if (postbyte & 0x10) {
 		ea = fetch_word_notrace(cpu, ea);
@@ -1486,7 +1445,7 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 	case 0: REG_X = reg; break;
 	case 1: REG_Y = reg; break;
 	case 2: REG_U = reg; break;
-	case 3: REG_S = reg; break;
+	case 3: cpu->nmi_armed |= (REG_S != reg); REG_S = reg; break;
 	}
 	return ea;
 }
@@ -1494,7 +1453,7 @@ static uint16_t ea_indexed(struct MC6809 *cpu) {
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 /*
- * Interrupt handling
+ * Interrupt handling, hooks
  */
 
 static void push_irq_registers(struct MC6809 *cpu) {
@@ -1529,12 +1488,26 @@ static void take_interrupt(struct MC6809 *cpu, uint8_t mask, uint16_t vec) {
 	REG_CC |= mask;
 	NVMA_CYCLE;
 	cpu->state = mc6809_state_irq_reset_vector;
+#ifdef TRACE
+	cpu->trace_next_pc = vec;
+	cpu->trace_nbytes = 0;
+#endif
 	REG_PC = fetch_word(cpu, vec);
 	cpu->state = mc6809_state_label_a;
 	NVMA_CYCLE;
+#ifdef TRACE
+	if (UNLIKELY(logging.trace_cpu)) {
+		mc6809_trace_vector(cpu->tracer, vec, cpu->trace_nbytes, cpu->trace_bytes);
+	}
+#endif
 }
 
 static void instruction_posthook(struct MC6809 *cpu) {
+#ifdef TRACE
+	if (UNLIKELY(logging.trace_cpu)) {
+		mc6809_trace_instruction(cpu->tracer, cpu->trace_pc, cpu->trace_nbytes, cpu->trace_bytes);
+	}
+#endif
 	DELEGATE_SAFE_CALL(cpu->debug_cpu.instruction_posthook);
 }
 
@@ -1544,7 +1517,7 @@ static void instruction_posthook(struct MC6809 *cpu) {
  * ALU operations
  */
 
-/* Illegal 6809 8-bit arithmetic operations */
+// Illegal 6809 8-bit arithmetic operations
 
 static uint8_t op_discard(struct MC6809 *cpu, uint8_t a, uint8_t b) {
 	(void)b;
