@@ -41,6 +41,9 @@
 #define PART_SER_PART (1)
 #define PART_SER_DATA (2)
 
+extern const struct partdb_entry clock_group_part;
+
+extern const struct machine_partdb_entry dragon32_part;
 extern const struct machine_partdb_entry dragon32_part;
 extern const struct machine_partdb_entry dragon64_part;
 extern const struct machine_partdb_entry dragonpro_part;
@@ -73,6 +76,7 @@ extern const struct partdb_entry mc6821_part;
 extern const struct partdb_entry mc6847_part;
 extern const struct partdb_entry mc6847t1_part;
 extern const struct partdb_entry mc6883_part;
+extern const struct partdb_entry samx8_part;
 extern const struct partdb_entry mos6551_part;
 extern const struct partdb_entry sn76489_part;
 extern const struct partdb_entry spi65_part;
@@ -88,6 +92,8 @@ extern const struct partdb_entry spi_sdcard_part;
 extern const struct cart_partdb_entry mcx128_part;
 
 const struct partdb_entry *partdb[] = {
+	&clock_group_part,
+
 #ifdef WANT_MACHINE_ARCH_DRAGON
 	(const struct partdb_entry *)&dragon32_part,
 	(const struct partdb_entry *)&dragon64_part,
@@ -140,6 +146,7 @@ const struct partdb_entry *partdb[] = {
 
 #ifdef WANT_PART_MC6883
 	&mc6883_part,
+	&samx8_part,
 #endif
 
 #ifdef WANT_PART_MC6847
@@ -489,10 +496,8 @@ void intf_init0(struct intf *i, struct part *p0, void *p0_idata, const char *nam
 
 void intf_free(struct intf *i) {
 	intf_detach(i);
-	if (i->name) {
-		free(i->name);
-		i->name = NULL;
-	}
+	free(i->name);
+	i->name = NULL;
 	if (i->free) {
 		i->free(i);
 	} else {
