@@ -2,7 +2,7 @@
  *
  *  \brief Printing to file or pipe
  *
- *  \copyright Copyright 2011-2016 Ciaran Anscomb
+ *  \copyright Copyright 2011-2024 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -21,20 +21,20 @@
 
 #include "delegate.h"
 
-struct machine;
+enum {
+	PRINTER_DESTINATION_NONE,
+	PRINTER_DESTINATION_FILE,
+	PRINTER_DESTINATION_PIPE,
+};
 
 struct printer_interface {
 	DELEGATE_T1(void, bool) signal_ack;
 };
 
-struct printer_interface *printer_interface_new(struct machine *m);
+struct printer_interface *printer_interface_new(void);
 void printer_interface_free(struct printer_interface *pi);
 void printer_reset(struct printer_interface *pi);
 
-void printer_open_file(struct printer_interface *pi, const char *filename);
-void printer_open_pipe(struct printer_interface *pi, const char *command);
-void printer_close(struct printer_interface *pi);
-
 void printer_flush(struct printer_interface *pi);
 void printer_strobe(struct printer_interface *pi, _Bool strobe, int data);
 _Bool printer_busy(struct printer_interface *pi);
