@@ -113,7 +113,7 @@ int read_snapshot(const char *filename) {
 		}
 	}
 	char *fncopy = xstrdup(filename);
-	event_queue_auto(&UI_EVENT_LIST, DELEGATE_AS0(void, do_retry_read_snapshot, fncopy), 1);
+	event_queue_auto(UI_EVENT_LIST, DELEGATE_AS0(void, do_retry_read_snapshot, fncopy), 1);
 	return -1;
 #endif
 	return 0;
@@ -315,7 +315,7 @@ static void old_set_registers(struct machine *m, uint8_t *regs) {
 	cpu->nmi = 0;
 	cpu->firq = 0;
 	cpu->irq = 0;
-	cpu->state = MC6809_COMPAT_STATE_NORMAL;
+	cpu->state = mc6809_state_label_a;
 	cpu->nmi_armed = 0;
 }
 
@@ -385,8 +385,7 @@ static int read_v1_snapshot(const char *filename) {
 				if (size < 1) break;
 				tmp = fs_read_uint8(fd);
 				tmp %= 4;
-				if (mc->architecture)
-					free(mc->architecture);
+				free(mc->architecture);
 				mc->architecture = xstrdup(old_arch_mapping[tmp]);
 				xroar_configure_machine(mc);
 				xroar.machine->reset(xroar.machine, RESET_HARD);
@@ -438,11 +437,11 @@ static int read_v1_snapshot(const char *filename) {
 						wait_for_interrupt = fs_read_uint8(fd);
 						skip_register_push = fs_read_uint8(fd);
 						if (wait_for_interrupt && skip_register_push) {
-							cpu->state = MC6809_COMPAT_STATE_CWAI;
+							cpu->state = mc6809_state_dispatch_irq;
 						} else if (wait_for_interrupt) {
-							cpu->state = MC6809_COMPAT_STATE_SYNC;
+							cpu->state = mc6809_state_sync;
 						} else {
-							cpu->state = MC6809_COMPAT_STATE_NORMAL;
+							cpu->state = mc6809_state_label_a;
 						}
 						size--;
 					} else {
@@ -623,7 +622,7 @@ static int read_v1_snapshot(const char *filename) {
 						LOG_MOD_WARN("snapshot/v1", "read: SAM not found - skipping SAM registers chunk\n");
 						break;
 					}
-					mc6883_set_register(sam, tmp);
+					sam->set_register(sam, tmp);
 				}
 				break;
 
@@ -679,20 +678,15 @@ static int read_v1_snapshot(const char *filename) {
 					if (!cart_config) {
 						cart_config = cart_config_new();
 					}
-					if (cart_config->name)
-						free(cart_config->name);
+					free(cart_config->name);
 					cart_config->name = name;
-					if (cart_config->description)
-						free(cart_config->description);
+					free(cart_config->description);
 					cart_config->description = desc;
-					if (cart_config->type)
-						free(cart_config->type);
+					free(cart_config->type);
 					cart_config->type = type;
-					if (cart_config->rom)
-						free(cart_config->rom);
+					free(cart_config->rom);
 					cart_config->rom = rom;
-					if (cart_config->rom2)
-						free(cart_config->rom2);
+					free(cart_config->rom2);
 					cart_config->rom2 = rom2;
 					cart_config->becker_port = fs_read_uint8(fd);
 					cart_config->autorun = fs_read_uint8(fd);
