@@ -2,7 +2,7 @@
  *
  *  \brief Video renderers.
  *
- *  \copyright Copyright 2003-2023 Ciaran Anscomb
+ *  \copyright Copyright 2003-2024 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -38,29 +38,57 @@
 
 #include "colourspace.h"
 #include "filter.h"
+#include "messenger.h"
 #include "ntsc.h"
 #include "vo_render.h"
 #include "xroar.h"
 
 #define MAX_FILTER_ORDER (15)
 
-const char *vo_render_fs_name[NUM_VO_RENDER_FS] = {
-	"14.31818 MHz",
-	"14.218 MHz",
-	"14.23753 MHz",
+// Pixel formats supported
+
+struct xconfig_enum vo_pixel_fmt_list[] = {
+	{ XC_ENUM_INT("rgba8", VO_RENDER_FMT_RGBA8, "RGBA 8-8-8-8, byte order") },
+	{ XC_ENUM_INT("argb8", VO_RENDER_FMT_ARGB8, "ARGB 8-8-8-8, byte order") },
+	{ XC_ENUM_INT("bgra8", VO_RENDER_FMT_BGRA8, "BGRA 8-8-8-8, byte order") },
+	{ XC_ENUM_INT("abgr8", VO_RENDER_FMT_ABGR8, "ABGR 8-8-8-8, byte order") },
+	{ XC_ENUM_INT("rgb565", VO_RENDER_FMT_RGB565, "RGB 5-6-5") },
+	{ XC_ENUM_INT("rgba4", VO_RENDER_FMT_RGBA4, "RGBA 4-4-4") },
+	{ XC_ENUM_INT("rgba32", VO_RENDER_FMT_RGBA32, "RGBA 8-8-8-8, host 32-bit packing") },
+	{ XC_ENUM_INT("argb32", VO_RENDER_FMT_ARGB32, "ARGB 8-8-8-8, host 32-bit packing") },
+	{ XC_ENUM_INT("bgra32", VO_RENDER_FMT_BGRA32, "BGRA 8-8-8-8, host 32-bit packing") },
+	{ XC_ENUM_INT("abgr32", VO_RENDER_FMT_ABGR32, "ABGR 8-8-8-8, host 32-bit packing") },
+	{ XC_ENUM_END() }
 };
 
-const char *vo_render_fsc_name[NUM_VO_RENDER_FSC] = {
-	"4.43361875 MHz",
-	"3.579545 MHz",
+// Pixel rates - used as sampling frequency when filtering
+
+struct xconfig_enum vo_render_fs_list[] = {
+	{ XC_ENUM_INT("ntsc", VO_RENDER_FS_14_31818, "14.31818 MHz") },
+	{ XC_ENUM_INT("pal-dragon", VO_RENDER_FS_14_218, "14.218 MHz") },
+	{ XC_ENUM_INT("pal-coco", VO_RENDER_FS_14_23753, "14.23753 MHz") },
+	{ XC_ENUM_END() }
 };
 
-const char *vo_render_system_name[NUM_VO_RENDER_SYSTEM] = {
-	"PAL-I",
-	"PAL-M",
-	"NTSC",
+// Colour subcarrier frequencies
+
+struct xconfig_enum vo_render_fsc_list[] = {
+	{ XC_ENUM_INT("pal", VO_RENDER_FSC_4_43361875, "4.43361875 MHz") },
+	{ XC_ENUM_INT("ntsc", VO_RENDER_FSC_3_579545, "3.579545 MHz") },
+	{ XC_ENUM_END() }
 };
 
+// Colour systems
+
+struct xconfig_enum vo_render_system_list[] = {
+	{ XC_ENUM_INT("pal-i", VO_RENDER_SYSTEM_PAL_I, "PAL-I") },
+	{ XC_ENUM_INT("pal-m", VO_RENDER_SYSTEM_PAL_M, "PAL-M") },
+	{ XC_ENUM_INT("ntsc", VO_RENDER_SYSTEM_NTSC, "NTSC") },
+	{ XC_ENUM_END() }
+};
+
+// Map (Fs, Fsc) to sets of ratios used to generate burst tables
+
 static const struct {
 	const int tmax;     // number of samples at F(s)
 	const int ncycles;  // number of cycles at F(sc)
@@ -88,6 +116,58 @@ static const double vo_render_fs_mhz[NUM_VO_RENDER_FS] = {
         14.31818, 14.218, 14.23753
 };
 
+// 2-bit cross-colour LUT
+
+static const uint8_t cmp_lut_2bit[2][4][3] = {
+	{
+		{ 0x00, 0x00, 0x00 }, { 0x00, 0x80, 0xff },
+		{ 0xff, 0x80, 0x00 }, { 0xff, 0xff, 0xff },
+	}, {
+		{ 0x00, 0x00, 0x00 }, { 0xff, 0x80, 0x00 },
+		{ 0x00, 0x80, 0xff }, { 0xff, 0xff, 0xff }
+	}
+};
+
+// 5-bit cross-colour LUT
+
+static const uint8_t cmp_lut_5bit[2][32][3] = {
+	{
+		{ 0x00, 0x00, 0x00 }, { 0x00, 0x00, 0x00 },
+		{ 0x00, 0x32, 0x78 }, { 0x00, 0x28, 0x00 },
+		{ 0xff, 0x8c, 0x64 }, { 0xff, 0x8c, 0x64 },
+		{ 0xff, 0xd2, 0xff }, { 0xff, 0xf0, 0xc8 },
+		{ 0x00, 0x32, 0x78 }, { 0x00, 0x00, 0x3c },
+		{ 0x00, 0x80, 0xff }, { 0x00, 0x80, 0xff },
+		{ 0xd2, 0xff, 0xd2 }, { 0xff, 0xff, 0xff },
+		{ 0x64, 0xf0, 0xff }, { 0xff, 0xff, 0xff },
+		{ 0x3c, 0x00, 0x00 }, { 0x3c, 0x00, 0x00 },
+		{ 0x00, 0x00, 0x00 }, { 0x00, 0x28, 0x00 },
+		{ 0xff, 0x80, 0x00 }, { 0xff, 0x80, 0x00 },
+		{ 0xff, 0xff, 0xff }, { 0xff, 0xf0, 0xc8 },
+		{ 0x28, 0x00, 0x28 }, { 0x28, 0x00, 0x28 },
+		{ 0x00, 0x80, 0xff }, { 0x00, 0x80, 0xff },
+		{ 0xff, 0xf0, 0xc8 }, { 0xff, 0xf0, 0xc8 },
+		{ 0xff, 0xff, 0xff }, { 0xff, 0xff, 0xff },
+	}, {
+		{ 0x00, 0x00, 0x00 }, { 0x00, 0x00, 0x00 },
+		{ 0xb4, 0x3c, 0x1e }, { 0x28, 0x00, 0x28 },
+		{ 0x46, 0xc8, 0xff }, { 0x46, 0xc8, 0xff },
+		{ 0xd2, 0xff, 0xd2 }, { 0x64, 0xf0, 0xff },
+		{ 0xb4, 0x3c, 0x1e }, { 0x3c, 0x00, 0x00 },
+		{ 0xff, 0x80, 0x00 }, { 0xff, 0x80, 0x00 },
+		{ 0xff, 0xd2, 0xff }, { 0xff, 0xff, 0xff },
+		{ 0xff, 0xf0, 0xc8 }, { 0xff, 0xff, 0xff },
+		{ 0x00, 0x00, 0x3c }, { 0x00, 0x00, 0x3c },
+		{ 0x00, 0x00, 0x00 }, { 0x28, 0x00, 0x28 },
+		{ 0x00, 0x80, 0xff }, { 0x00, 0x80, 0xff },
+		{ 0xff, 0xff, 0xff }, { 0x64, 0xf0, 0xff },
+		{ 0x00, 0x28, 0x00 }, { 0x00, 0x28, 0x00 },
+		{ 0xff, 0x80, 0x00 }, { 0xff, 0x80, 0x00 },
+		{ 0x64, 0xf0, 0xff }, { 0x64, 0xf0, 0xff },
+		{ 0xff, 0xff, 0xff }, { 0xff, 0xff, 0xff },
+	}
+};
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 #define VR_PTYPE uint8_t
@@ -108,12 +188,19 @@ static const double vo_render_fs_mhz[NUM_VO_RENDER_FS] = {
 #undef VR_PTYPE
 #undef VR_SUFFIX
 
-static uint32_t map_rgba8(struct vo_render *vr, int R, int G, int B);
-static uint32_t map_argb8(struct vo_render *vr, int R, int G, int B);
-static uint32_t map_bgra8(struct vo_render *vr, int R, int G, int B);
-static uint32_t map_abgr8(struct vo_render *vr, int R, int G, int B);
-static uint16_t map_rgba4(struct vo_render *vr, int R, int G, int B);
-static uint16_t map_rgb565(struct vo_render *vr, int R, int G, int B);
+static uint32_t map_rgba8(int R, int G, int B);
+static uint32_t map_argb8(int R, int G, int B);
+static uint32_t map_bgra8(int R, int G, int B);
+static uint32_t map_abgr8(int R, int G, int B);
+static uint16_t map_rgba4(int R, int G, int B);
+static uint16_t map_rgb565(int R, int G, int B);
+
+static int_xyz unmap_rgba8(uint32_t s);
+static int_xyz unmap_argb8(uint32_t s);
+static int_xyz unmap_bgra8(uint32_t s);
+static int_xyz unmap_abgr8(uint32_t s);
+static int_xyz unmap_rgba4(uint16_t s);
+static int_xyz unmap_rgb565(uint16_t s);
 
 static void render_rgba8(struct vo_render *vr, int_xyz *src, void *dest, unsigned npixels);
 static void render_argb8(struct vo_render *vr, int_xyz *src, void *dest, unsigned npixels);
@@ -124,6 +211,16 @@ static void render_rgb565(struct vo_render *vr, int_xyz *src, void *dest, unsign
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+static void update_cmp_system(struct vo_render *vr);
+static void vr_ui_set_cmp_fs(void *, int tag, void *smsg);
+static void vr_ui_set_cmp_fsc(void *, int tag, void *smsg);
+static void vr_ui_set_cmp_system(void *, int tag, void *smsg);
+static void vr_ui_set_cmp_colour_killer(void *, int tag, void *smsg);
+static void vr_ui_set_ntsc_scaling(void *, int tag, void *smsg);
+static void vr_ui_set_brightness(void *, int tag, void *smsg);
+static void vr_ui_set_contrast(void *, int tag, void *smsg);
+static void vr_ui_set_saturation(void *, int tag, void *smsg);
+static void vr_ui_set_hue(void *, int tag, void *smsg);
 static void update_gamma_table(struct vo_render *vr);
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -134,32 +231,32 @@ struct vo_render *vo_render_new(int fmt) {
 	struct vo_render *vr = NULL;
 	switch (fmt) {
 	case VO_RENDER_FMT_RGBA8:
-		vr = renderer_new_uint32(map_rgba8);
+		vr = renderer_new_uint32(map_rgba8, unmap_rgba8);
 		vr->render_rgb = render_rgba8;
 		break;
 
 	case VO_RENDER_FMT_ARGB8:
-		vr = renderer_new_uint32(map_argb8);
+		vr = renderer_new_uint32(map_argb8, unmap_argb8);
 		vr->render_rgb = render_argb8;
 		break;
 
 	case VO_RENDER_FMT_BGRA8:
-		vr = renderer_new_uint32(map_bgra8);
+		vr = renderer_new_uint32(map_bgra8, unmap_bgra8);
 		vr->render_rgb = render_bgra8;
 		break;
 
 	case VO_RENDER_FMT_ABGR8:
-		vr = renderer_new_uint32(map_abgr8);
+		vr = renderer_new_uint32(map_abgr8, unmap_abgr8);
 		vr->render_rgb = render_abgr8;
 		break;
 
 	case VO_RENDER_FMT_RGBA4:
-		vr = renderer_new_uint16(map_rgba4);
+		vr = renderer_new_uint16(map_rgba4, unmap_rgba4);
 		vr->render_rgb = render_rgba4;
 		break;
 
 	case VO_RENDER_FMT_RGB565:
-		vr = renderer_new_uint16(map_rgb565);
+		vr = renderer_new_uint16(map_rgb565, unmap_rgb565);
 		vr->render_rgb = render_rgb565;
 		break;
 
@@ -170,18 +267,33 @@ struct vo_render *vo_render_new(int fmt) {
 	if (!vr)
 		return NULL;
 
+	vr->msgr_client_id = messenger_client_register();
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_cmp_fs, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_cmp_fs, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_cmp_fsc, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_cmp_fsc, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_cmp_system, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_cmp_system, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_cmp_colour_killer, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_cmp_colour_killer, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_ntsc_scaling, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_ntsc_scaling, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_brightness, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_brightness, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_contrast, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_contrast, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_saturation, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_saturation, vr));
+	ui_messenger_preempt_group(vr->msgr_client_id, ui_tag_hue, MESSENGER_NOTIFY_DELEGATE(vr_ui_set_hue, vr));
+
 	// Sensible defaults
-	vo_render_set_cmp_fs(vr, 1, VO_RENDER_FS_14_31818);
-	vo_render_set_cmp_fsc(vr, 1, VO_RENDER_FSC_4_43361875);
-	vo_render_set_cmp_system(vr, 1, VO_RENDER_SYSTEM_PAL_I);
+	vr->cmp.fs = VO_RENDER_FS_14_31818;
+	vr->cmp.fsc = VO_RENDER_FSC_4_43361875;
+	vr->cmp.system = VO_RENDER_SYSTEM_PAL_I;
+	update_cmp_system(vr);
 
 	vr->cmp.cha_phase = M_PI/2.;  // default 90°
-	vr->viewport.new_x = 190;
-	vr->viewport.new_y = 14;
-	vr->viewport.x = 190;
-	vr->viewport.y = 14;
+
+	// Sensible viewport defaults.  Will be updated to centre the display
+	// in short order.  Default X offset is enough to apply two filters
+	// without having to worry about underflow.
+	vr->viewport.x = 2 * MAX_FILTER_ORDER;
 	vr->viewport.w = 640;
 	vr->viewport.h = 240;
+	vo_render_set_active_area(vr, 254, 63, 512, 512);
+
 	vr->t = 0;
 	vr->tmax = 4;
 	vr->brightness = 50;
@@ -190,15 +302,21 @@ struct vo_render *vo_render_new(int fmt) {
 	vr->cmp.phase_offset = 0;
 
 	// Populate LUTs
-	for (int i = 0; i < 2; i++) {
+	for (unsigned i = 0; i < 2; ++i) {
 		// 2-bit LUT NTSC cross-colour
-		for (int j = 0; j < 4; j++) {
-			vr->set_palette_entry(vr, VO_RENDER_PALETTE_CMP_2BIT, i*4+j, vo_cmp_lut_2bit[i][j][0], vo_cmp_lut_2bit[i][j][1], vo_cmp_lut_2bit[i][j][2]);
+		for (unsigned j = 0; j < 4; ++j) {
+			vr->set_palette_entry(vr, VO_RENDER_PALETTE_CMP_2BIT, i*4+j,
+					      cmp_lut_2bit[i][j][0],
+					      cmp_lut_2bit[i][j][1],
+					      cmp_lut_2bit[i][j][2]);
 		}
 		// 5-bit LUT NTSC cross-colour
 		// TODO: generate this using available NTSC decoding
-		for (int j = 0; j < 32; j++) {
-			vr->set_palette_entry(vr, VO_RENDER_PALETTE_CMP_5BIT, i*32+j, vo_cmp_lut_5bit[i][j][0], vo_cmp_lut_5bit[i][j][1], vo_cmp_lut_5bit[i][j][2]);
+		for (unsigned j = 0; j < 32; ++j) {
+			vr->set_palette_entry(vr, VO_RENDER_PALETTE_CMP_5BIT, i*32+j,
+					      cmp_lut_5bit[i][j][0],
+					      cmp_lut_5bit[i][j][1],
+					      cmp_lut_5bit[i][j][2]);
 		}
 	}
 	update_gamma_table(vr);
@@ -209,6 +327,7 @@ struct vo_render *vo_render_new(int fmt) {
 // Free renderer
 
 void vo_render_free(struct vo_render *vr) {
+	messenger_client_unregister(vr->msgr_client_id);
 	free(vr->cmp.burst);
 	if (vr->cmp.mod.ufilter.coeff) {
 		free(vr->cmp.mod.ufilter.coeff - MAX_FILTER_ORDER);
@@ -234,36 +353,54 @@ extern inline void vo_render_set_buffer(struct vo_render *vr, void *buffer);
 
 // Pack R,G,B into a particular pixel format
 
-static uint32_t map_rgba8(struct vo_render *vr, int R, int G, int B) {
-	(void)vr;
+static uint32_t map_rgba8(int R, int G, int B) {
 	return (R << 24) | (G << 16) | (B << 8) | 0xff;
 }
 
-static uint32_t map_argb8(struct vo_render *vr, int R, int G, int B) {
-	(void)vr;
+static uint32_t map_argb8(int R, int G, int B) {
 	return 0xff000000 | (R << 16) | (G << 8) | (B << 0);
 }
 
-static uint32_t map_bgra8(struct vo_render *vr, int R, int G, int B) {
-	(void)vr;
+static uint32_t map_bgra8(int R, int G, int B) {
 	return (B << 24) | (G << 16) | (R << 8) | 0xff;
 }
 
-static uint32_t map_abgr8(struct vo_render *vr, int R, int G, int B) {
-	(void)vr;
+static uint32_t map_abgr8(int R, int G, int B) {
 	return 0xff000000 | (B << 16) | (G << 8) | (R << 0);
 }
 
-static uint16_t map_rgba4(struct vo_render *vr, int R, int G, int B) {
-	(void)vr;
+static uint16_t map_rgba4(int R, int G, int B) {
 	return ((R & 0xf0) << 8) | ((G & 0xf0) << 4) | (B & 0xf0) | 0x0f;
 }
 
-static uint16_t map_rgb565(struct vo_render *vr, int R, int G, int B) {
-	(void)vr;
+static uint16_t map_rgb565(int R, int G, int B) {
 	return ((R & 0xf8) << 8) | ((G & 0xfc) << 3) | ((B & 0xf8) >> 3);
 }
 
+static int_xyz unmap_rgba8(uint32_t s) {
+	return (int_xyz){.x = (s >> 24) & 0xff, .y = (s >> 16) & 0xff, .z = (s >> 8) & 0xff};
+}
+
+static int_xyz unmap_argb8(uint32_t s) {
+	return (int_xyz){.x = (s >> 16) & 0xff, .y = (s >> 8) & 0xff, .z = s & 0xff};
+}
+
+static int_xyz unmap_bgra8(uint32_t s) {
+	return (int_xyz){.x = (s >> 8) & 0xff, .y = (s >> 16) & 0xff, .z = (s >> 24) & 0xff};
+}
+
+static int_xyz unmap_abgr8(uint32_t s) {
+	return (int_xyz){.x = s & 0xff, .y = (s >> 8) & 0xff, .z = (s >> 16) & 0xff};
+}
+
+static int_xyz unmap_rgba4(uint16_t s) {
+	return (int_xyz){.x = (s >> 8) & 0xf0, .y = (s >> 4) & 0xf0, .z = s & 0xf0};
+}
+
+static int_xyz unmap_rgb565(uint16_t s) {
+	return (int_xyz){.x = (s >> 8) & 0xf8, .y = (s >> 3) & 0xfc, .z = (s >> 3) & 0xf8};
+}
+
 // Render a line of RGB data into a particular pixel format.  The calls to
 // map_* (defined above) should get inlined, as they are usually trivial.
 
@@ -273,7 +410,7 @@ static void render_rgba8(struct vo_render *vr, int_xyz *src, void *dest, unsigne
 		int R = vr->ungamma[int_clamp_u8(src->x)];
 		int G = vr->ungamma[int_clamp_u8(src->y)];
 		int B = vr->ungamma[int_clamp_u8(src->z)];
-		*(d++) = map_rgba8(vr, R, G, B);
+		*(d++) = map_rgba8(R, G, B);
 	}
 }
 
@@ -283,7 +420,7 @@ static void render_argb8(struct vo_render *vr, int_xyz *src, void *dest, unsigne
 		int R = vr->ungamma[int_clamp_u8(src->x)];
 		int G = vr->ungamma[int_clamp_u8(src->y)];
 		int B = vr->ungamma[int_clamp_u8(src->z)];
-		*(d++) = map_argb8(vr, R, G, B);
+		*(d++) = map_argb8(R, G, B);
 	}
 }
 
@@ -293,7 +430,7 @@ static void render_bgra8(struct vo_render *vr, int_xyz *src, void *dest, unsigne
 		int R = vr->ungamma[int_clamp_u8(src->x)];
 		int G = vr->ungamma[int_clamp_u8(src->y)];
 		int B = vr->ungamma[int_clamp_u8(src->z)];
-		*(d++) = map_bgra8(vr, R, G, B);
+		*(d++) = map_bgra8(R, G, B);
 	}
 }
 
@@ -303,7 +440,7 @@ static void render_abgr8(struct vo_render *vr, int_xyz *src, void *dest, unsigne
 		int R = vr->ungamma[int_clamp_u8(src->x)];
 		int G = vr->ungamma[int_clamp_u8(src->y)];
 		int B = vr->ungamma[int_clamp_u8(src->z)];
-		*(d++) = map_abgr8(vr, R, G, B);
+		*(d++) = map_abgr8(R, G, B);
 	}
 }
 
@@ -313,7 +450,7 @@ static void render_rgba4(struct vo_render *vr, int_xyz *src, void *dest, unsigne
 		int R = vr->ungamma[int_clamp_u8(src->x)];
 		int G = vr->ungamma[int_clamp_u8(src->y)];
 		int B = vr->ungamma[int_clamp_u8(src->z)];
-		*(d++) = map_rgba4(vr, R, G, B);
+		*(d++) = map_rgba4(R, G, B);
 	}
 }
 
@@ -323,7 +460,7 @@ static void render_rgb565(struct vo_render *vr, int_xyz *src, void *dest, unsign
 		int R = vr->ungamma[int_clamp_u8(src->x)];
 		int G = vr->ungamma[int_clamp_u8(src->y)];
 		int B = vr->ungamma[int_clamp_u8(src->z)];
-		*(d++) = map_rgb565(vr, R, G, B);
+		*(d++) = map_rgb565(R, G, B);
 	}
 }
 
@@ -384,7 +521,7 @@ static void update_cmp_palette(struct vo_render *vr, uint8_t c) {
 	// Track "black or white" for simple artefact renderers
 	if (y > 0.85 && fabs(b_y) < 0.10 && fabs(r_y) < 0.10) {
 		vr->cmp.is_black_or_white[c] = 3;
-	} else if (y < 0.20 && fabs(b_y) < 0.10 && fabs(r_y) < 0.10) {
+	} else if (y < 0.25 && fabs(b_y) < 0.10 && fabs(r_y) < 0.10) {
 		vr->cmp.is_black_or_white[c] = 2;
 	} else {
 		vr->cmp.is_black_or_white[c] = 0;
@@ -585,85 +722,109 @@ static void update_cmp_system(struct vo_render *vr) {
 	                     vr->cmp.mod.ufilter.order : vr->cmp.mod.vfilter.order;
 	vr->cmp.demod.corder = (vr->cmp.demod.ufilter.order > vr->cmp.demod.vfilter.order) ?
 	                       vr->cmp.demod.ufilter.order : vr->cmp.demod.vfilter.order;
+	vr->cmp.demod.morder = (vr->cmp.demod.corder > vr->cmp.demod.yfilter.order) ?
+	                       vr->cmp.demod.corder : vr->cmp.demod.yfilter.order;
 
 	for (unsigned i = 0; i < vr->cmp.nbursts; i++) {
 		update_cmp_burst(vr, i);
 	}
 }
 
+// Update viewport offset based on viewport dimensions and active area
+
+static void update_viewport(struct vo_render *vr) {
+	vr->viewport.new_x = vr->active_area.x - (vr->viewport.w - vr->active_area.w) / 2;
+	vr->viewport.new_y = vr->active_area.y - (vr->viewport.h - vr->active_area.h) / 2;
+}
+
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 // Used by UI to adjust viewing parameters
 
-// Set brightness
-//     int brightness;  // 0-100
+// Set viewport dimensions
+//     int w, h;  // dimensions
 
-void vo_render_set_brightness(void *sptr, int value) {
+void vo_render_set_viewport(struct vo_render *vr, int w, int h) {
+	vr->viewport.w = w;
+	vr->viewport.h = h;
+	update_viewport(vr);
+}
+
+// Set 60Hz scaling
+//     value = enabled
+
+static void vr_ui_set_ntsc_scaling(void *sptr, int tag, void *smsg) {
 	struct vo_render *vr = sptr;
-	if (value < 0) value = 0;
-	if (value > 100) value = 100;
-	vr->brightness = value;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_ntsc_scaling);
+
+	vr->ntsc_scaling = ui_msg_adjust_value_range(uimsg, vr->ntsc_scaling, 0, 0, 1,
+						     UI_ADJUST_FLAG_CYCLE);
+}
+
+// Set brightness
+//     value = brightness (0-100)
+
+static void vr_ui_set_brightness(void *sptr, int tag, void *smsg) {
+	struct vo_render *vr = sptr;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_brightness);
+
+	vr->brightness = ui_msg_adjust_value_range(uimsg, vr->brightness, 50, 0, 100, 0);
 	for (unsigned c = 0; c < 256; c++) {
 		update_cmp_palette(vr, c);
 		update_rgb_palette(vr, c);
 	}
 	update_gamma_table(vr);
-	if (xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_brightness, value, NULL);
-	}
 }
 
 // Set contrast
-//     int contrast;  // 0-100
+//     value = contrast (0-100)
 
-void vo_render_set_contrast(void *sptr, int value) {
+static void vr_ui_set_contrast(void *sptr, int tag, void *smsg) {
 	struct vo_render *vr = sptr;
-	if (value < 0) value = 0;
-	if (value > 100) value = 100;
-	vr->contrast = value;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_contrast);
+
+	vr->contrast = ui_msg_adjust_value_range(uimsg, vr->contrast, 50, 0, 100, 0);
 	for (unsigned c = 0; c < 256; c++) {
 		update_cmp_palette(vr, c);
 		update_rgb_palette(vr, c);
 	}
 	update_gamma_table(vr);
-	if (xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_contrast, value, NULL);
-	}
 }
 
 // Set colour saturation
-//     int saturation;  // 0-100
+//     value = saturation (0-100)
 
-void vo_render_set_saturation(void *sptr, int value) {
+static void vr_ui_set_saturation(void *sptr, int tag, void *smsg) {
 	struct vo_render *vr = sptr;
-	if (value < 0) value = 0;
-	if (value > 100) value = 100;
-	vr->saturation = value;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_saturation);
+
+	vr->saturation = ui_msg_adjust_value_range(uimsg, vr->saturation, 50, 0, 100, 0);
 	vr->cmp.demod.saturation = (int)(((double)vr->saturation * 512.) / 100.);
 	for (unsigned c = 0; c < 256; c++) {
 		update_cmp_palette(vr, c);
 	}
-	if (xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_saturation, value, NULL);
-	}
 }
 
 // Set hue
-//     int hue;  // -179 to +180
+//     value = hue (-179 to +180)
 
-void vo_render_set_hue(void *sptr, int value) {
+static void vr_ui_set_hue(void *sptr, int tag, void *smsg) {
 	struct vo_render *vr = sptr;
-	value = ((value + 179) % 360) - 179;
-	vr->hue = value;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_hue);
+
+	vr->hue = ui_msg_adjust_value_range(uimsg, vr->hue, 0, -179, 180,
+					    UI_ADJUST_FLAG_CYCLE);
 	for (unsigned c = 0; c < 256; c++) {
 		update_cmp_palette(vr, c);
 	}
 	for (unsigned i = 0; i < vr->cmp.nbursts; i++) {
 		update_cmp_burst(vr, i);
 	}
-	if (xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_hue, value, NULL);
-	}
 }
 
 // Set cross-colour phase
@@ -685,67 +846,65 @@ void vo_render_set_cmp_phase(void *sptr, int value) {
 
 void vo_render_set_active_area(void *sptr, int x, int y, int w, int h) {
 	struct vo_render *vr = sptr;
-	int xoff = x - (640 - w) / 2;
-	int yoff = y - (240 - h) / 2;
-	vr->viewport.new_x = xoff;
-	vr->viewport.new_y = yoff;
+	vr->active_area.x = x;
+	vr->active_area.y = y;
+	vr->active_area.w = w;
+	vr->active_area.h = h;
+	update_viewport(vr);
 }
 
-// Set sampling frequency (equal to pixel rate) to one of VO_RENDER_FS_*
+// Set sampling frequency (equal to pixel rate)
+//     value = VO_RENDER_FS_*
 
-void vo_render_set_cmp_fs(struct vo_render *vr, _Bool notify, int fs) {
-	if (!vr)
-		return;
-	if (fs < 0 || fs >= NUM_VO_RENDER_FS) {
-		fs = VO_RENDER_FS_14_31818;
-	}
-	vr->cmp.fs = fs;
+static void vr_ui_set_cmp_fs(void *sptr, int tag, void *smsg) {
+	struct vo_render *vr = sptr;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_cmp_fs);
+
+	vr->cmp.fs = ui_msg_adjust_value_range(uimsg, vr->cmp.fs, VO_RENDER_FS_14_31818,
+					       VO_RENDER_FS_14_31818, VO_RENDER_FS_14_23753, 0);
 	update_cmp_system(vr);
-	if (notify && xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_cmp_fs, fs, NULL);
-	}
 }
 
-// Set chroma subcarrier frequency to one of VO_RENDER_FSC_*
+// Set chroma subcarrier frequency
+//     value = VO_RENDER_FSC_*
 
-void vo_render_set_cmp_fsc(struct vo_render *vr, _Bool notify, int fsc) {
-	if (!vr)
-		return;
-	if (fsc < 0 || fsc >= NUM_VO_RENDER_FSC) {
-		fsc = VO_RENDER_FSC_4_43361875;
-	}
-	vr->cmp.fsc = fsc;
+static void vr_ui_set_cmp_fsc(void *sptr, int tag, void *smsg) {
+	struct vo_render *vr = sptr;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_cmp_fsc);
+
+	vr->cmp.fsc = ui_msg_adjust_value_range(uimsg, vr->cmp.fsc, VO_RENDER_FSC_4_43361875,
+						VO_RENDER_FSC_4_43361875,
+						VO_RENDER_FSC_3_579545, 0);
 	update_cmp_system(vr);
-	if (notify && xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_cmp_fsc, fsc, NULL);
-	}
 }
 
-// Set colour system to one of VO_RENDER_SYSTEM_*
+// Set colour system
+//     value = VO_RENDER_SYSTEM_*
 
-void vo_render_set_cmp_system(struct vo_render *vr, _Bool notify, int system) {
-	if (!vr)
-		return;
-	if (system < 0 || system >= NUM_VO_RENDER_SYSTEM) {
-		system = VO_RENDER_SYSTEM_PAL_I;
-	}
-	vr->cmp.system = system;
+static void vr_ui_set_cmp_system(void *sptr, int tag, void *smsg) {
+	struct vo_render *vr = sptr;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_cmp_system);
+
+	vr->cmp.system = ui_msg_adjust_value_range(uimsg, vr->cmp.system,
+						   VO_RENDER_SYSTEM_PAL_I,
+						   VO_RENDER_SYSTEM_PAL_I,
+						   VO_RENDER_SYSTEM_NTSC, 0);
 	update_cmp_system(vr);
-	if (notify && xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_cmp_system, system, NULL);
-	}
 }
 
-// Set whether colour decoding is to be disabled when no colourburst indicated
-// (where supported)
+// Set whether colour decoding disabled when no colourburst indicated
+//     value = enabled
 
-void vo_render_set_cmp_colour_killer(struct vo_render *vr, _Bool notify, _Bool value) {
-	if (!vr)
-		return;
-	vr->cmp.colour_killer = value;
-	if (notify && xroar_ui_interface) {
-		DELEGATE_CALL(xroar_ui_interface->update_state, ui_tag_cmp_colour_killer, (int)value, NULL);
-	}
+static void vr_ui_set_cmp_colour_killer(void *sptr, int tag, void *smsg) {
+	struct vo_render *vr = sptr;
+	struct ui_state_message *uimsg = smsg;
+	assert(tag == ui_tag_cmp_colour_killer);
+
+	vr->cmp.colour_killer = ui_msg_adjust_value_range(uimsg, vr->cmp.colour_killer, 0,
+							  0, 1, UI_ADJUST_FLAG_CYCLE);
 }
 
 // Set how the chroma components relate to each other (in degrees)
@@ -834,6 +993,13 @@ void vo_render_vsync(void *sptr) {
 	if (!vr)
 		return;
 	vr->pixel = vr->buffer;
+
+	_Bool is_60hz = vr->ntsc_scaling && (vr->scanline < 288);
+	if (is_60hz != vr->is_60hz) {
+		vr->is_60hz = is_60hz;
+		DELEGATE_SAFE_CALL(vr->notify_frame_rate, is_60hz);
+	}
+
 	vr->scanline = 0;
 	vr->viewport.x = vr->viewport.new_x;
 	vr->viewport.y = vr->viewport.new_y;
@@ -923,7 +1089,7 @@ void vo_render_cmp_simulated(void *sptr, unsigned burstn, unsigned npixels, uint
 		vr->cmp.vswitch = !vswitch;
 
 	// Optionally apply lowpass filters to U and V.  Modulate results.
-	for (unsigned i = MAX_FILTER_ORDER; i < npixels - MAX_FILTER_ORDER; i++) {
+	for (unsigned i = vr->viewport.x - vr->cmp.demod.morder; i < (unsigned)(vr->viewport.x + vr->viewport.w + vr->cmp.demod.morder); i++) {
 		int c = data[i];
 		int py = vr->cmp.palette.y[c];
 
@@ -962,7 +1128,7 @@ void vo_render_cmp_simulated(void *sptr, unsigned burstn, unsigned npixels, uint
 
 	int_xyz rgb[1024];
 
-	for (unsigned i = MAX_FILTER_ORDER; i < npixels - MAX_FILTER_ORDER; i++) {
+	for (unsigned i = vr->viewport.x; i < (unsigned)(vr->viewport.x + vr->viewport.w); i++) {
 		int fy = 0;
 		int yorder = vr->cmp.demod.yfilter.order;
 		for (int ft = -yorder; ft <= yorder; ft++) {
