@@ -189,8 +189,8 @@ static void finish_init(void *sptr) {
 	// During initialisation, the UI event list is used to reschedule
 	// things that failed due to needing incomplete file transfers.
 	// Process it without checking against scheduled time.
-	while (UI_EVENT_LIST) {
-		event_dispatch_next(&UI_EVENT_LIST);
+	while (UI_EVENT_LIST->events) {
+		event_dispatch_next(UI_EVENT_LIST);
 	}
 
 	// Running the events could lead to more waiting...
@@ -280,9 +280,7 @@ static void wasm_ui_state_notify(void *sptr, int tag, void *smsg) {
 			char *fn = data ? xstrdup((const char *)data) : NULL;
 			char *bn = fn ? basename(fn) : NULL;
 			EM_ASM_({ ui_update_tape_input_filename($0); }, bn);
-			if (fn) {
-				free(fn);
-			}
+			free(fn);
 		}
 		break;
 
@@ -299,9 +297,7 @@ static void wasm_ui_state_notify(void *sptr, int tag, void *smsg) {
 				char *fn = disk->filename ? xstrdup(disk->filename) : NULL;
 				char *bn = fn ? basename(fn) : NULL;
 				EM_ASM_({ ui_update_disk_info($0, $1, $2, $3, $4, $5); }, value, bn, disk->write_back, disk->write_protect, disk->num_cylinders, disk->num_heads);
-				if (fn) {
-					free(fn);
-				}
+				free(fn);
 			} else {
 				EM_ASM_({ ui_update_disk_info($0, null, 0, 0, -1, 0); }, value);
 			}
@@ -768,7 +764,7 @@ void wasm_queue_message_value_event(int tag, int value) {
 	struct wasm_message_value *mv = xmalloc(sizeof(*mv));
 	mv->tag = tag;
 	mv->value = value;
-	event_queue_auto(&UI_EVENT_LIST, DELEGATE_AS0(void, do_message_value_event, mv), 1);
+	event_queue_auto(UI_EVENT_LIST, DELEGATE_AS0(void, do_message_value_event, mv), 1);
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -866,16 +862,12 @@ void wasm_set_machine_cart(const char *machine, const char *cart,
 	struct cart_config *cc = cart_config_by_name(cart);
 	if (cc) {
 		if (cart_rom) {
-			if (cc->rom) {
-				free(cc->rom);
-				cc->rom = NULL;
-			}
+			free(cc->rom);
+			cc->rom = NULL;
 			cc->rom = xstrdup(cart_rom);
 		}
-		if (cc->rom2) {
-			free(cc->rom2);
-			cc->rom2 = NULL;
-		}
+		free(cc->rom2);
+		cc->rom2 = NULL;
 		if (cart_rom2) {
 			cc->rom2 = xstrdup(cart_rom2);
 		}
@@ -935,7 +927,7 @@ void wasm_load_file(const char *filename, int type, int drive) {
 	ev->type = type;
 	ev->drive = drive;
 	wasm_wget(filename);
-	event_queue_auto(&UI_EVENT_LIST, DELEGATE_AS0(void, do_wasm_load_file, ev), 1);
+	event_queue_auto(UI_EVENT_LIST, DELEGATE_AS0(void, do_wasm_load_file, ev), 1);
 }
 
 // Submit BASIC commands
@@ -950,7 +942,7 @@ static void do_wasm_queue_basic(void *sptr) {
 void wasm_queue_basic(const char *string) {
 	char *text = xstrdup(string);
 	WASM_DEBUG("wasm_queue_basic(%s): queueing do_wasm_queue_basic()\n", string);
-	event_queue_auto(&UI_EVENT_LIST, DELEGATE_AS0(void, do_wasm_queue_basic, text), 1);
+	event_queue_auto(UI_EVENT_LIST, DELEGATE_AS0(void, do_wasm_queue_basic, text), 1);
 }
 
 // Keyboard interface
