@@ -2,7 +2,7 @@
  *
  *  \brief WD279x Floppy Drive Controller.
  *
- *  \copyright Copyright 2003-2024 Ciaran Anscomb
+ *  \copyright Copyright 2003-2025 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -126,8 +126,7 @@ static const char * const wd279x_type_name[4] = {
 
 #define NEXT_STATE(fdc,s,t) do { \
 		(fdc)->state = (s); \
-		(fdc)->state_event.at_tick = event_current_tick + (t); \
-		event_queue(&MACHINE_EVENT_LIST, &(fdc)->state_event); \
+		event_queue_dt(&(fdc)->state_event, (t)); \
 	} while (0)
 
 #define SET_STATE(fdc,s) (fdc)->state = (s)
@@ -202,7 +201,7 @@ static struct part *wd279x_allocate(void) {
 
 	*fdc = (struct WD279X){0};
 
-	event_init(&fdc->state_event, DELEGATE_AS0(void, wd279x_state_machine, fdc));
+	event_init(&fdc->state_event, MACHINE_EVENT_LIST, DELEGATE_AS0(void, wd279x_state_machine, fdc));
 	wd279x_disconnect(fdc);
 
 	return p;
@@ -234,7 +233,7 @@ static _Bool wd279x_finish(struct part *p) {
 	fdc->invert_data = (fdc->type == WD2791 || fdc->type == WD2795) ? 0xff : 0;
 
 	if (fdc->state_event.next == &fdc->state_event) {
-		event_queue(&MACHINE_EVENT_LIST, &fdc->state_event);
+		event_queue(&fdc->state_event);
 	}
 
 	return 1;
@@ -761,13 +760,12 @@ static void wd279x_state_machine(void *sptr) {
 
 
 		case WD279X_state_read_sector_2:
-			if (fdc->status_register & STATUS_DRQ) {
-				fdc->status_register |= STATUS_LOST_DATA;
-				if (logging.debug_fdc & LOG_FDC_DATA)
-					log_hexdump_flag(fdc->log_rsec_hex);
-				//RESET_DRQ(fdc);  // XXX
-			}
 			if (fdc->bytes_left > 0) {
+				if (fdc->status_register & STATUS_DRQ) {
+					fdc->status_register |= STATUS_LOST_DATA;
+					if (logging.debug_fdc & LOG_FDC_DATA)
+						log_hexdump_flag(fdc->log_rsec_hex);
+				}
 				fdc->data_register = wd279x_read_byte(fdc);
 				if (logging.debug_fdc & LOG_FDC_DATA)
 					log_hexdump_byte(fdc->log_rsec_hex, fdc->data_register);
