@@ -2,7 +2,7 @@
  *
  *  \brief Windows drive control window.
  *
- *  \copyright Copyright 2023 Ciaran Anscomb
+ *  \copyright Copyright 2024 Ciaran Anscomb
  *
  *  \licenseblock This file is part of XRoar, a Dragon/Tandy CoCo emulator.
  *
@@ -19,16 +19,9 @@
 #ifndef XROAR_WINDOWS_DRIVECONTROL_H_
 #define XROAR_WINDOWS_DRIVECONTROL_H_
 
-struct ui_sdl2_interface;
+struct ui_windows32_interface;
+struct uiw32_dialog;
 
-void windows32_dc_create_window(struct ui_sdl2_interface *uisdl2);
-void windows32_dc_show_window(struct ui_sdl2_interface *uisdl2);
-
-void windows32_dc_update_drive_disk(struct ui_sdl2_interface *uisdl2,
-				    int drive, const struct vdisk *disk);
-void windows32_dc_update_drive_write_enable(struct ui_sdl2_interface *uisdl2,
-					    int drive, _Bool write_enable);
-void windows32_dc_update_drive_write_back(struct ui_sdl2_interface *uisdl2,
-					  int drive, _Bool write_back);
+struct uiw32_dialog *uiw32_dc_dialog_new(struct ui_windows32_interface *);
 
 #endif
